/* color_map.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Routines for loading colors from .map and .fmap (fade-map) files.
 */

#include <stdio.h>
#include <string.h>

#include "config.h"
#include "color_map.h"
#include "util.h"
#include "constants.h"

extern path_info paths;

/* Copies at most "ncolors" colors from group "name" to "colors", and returns
 * the actual number of colors copied */
int 
copy_colors_from_map (GdkColor * colors, int ncolors, char * name,
		      GList * color_map_list)
{
    int i;
    color_group * g;
    GList *found = g_list_find_custom (color_map_list, name, color_map_find);

    g_assert (found != NULL);

    g = (color_group *)found->data;

    /* Fading color map */
    if (g->type == FMAP)
    {
	make_color_path_rgb (g, colors, &ncolors);
	return ncolors;
    }

    /* Normal color map - copy colors directly */
    else
    {
	for (i = 0; i < ncolors && i < g_list_length(g->colors); ++i)
	{
	    dcolor * c = (dcolor *)g_list_nth_data(g->colors, i);
	    colors[i].red   = c->r;
	    colors[i].green = c->g;
	    colors[i].blue  = c->b;
	}
	return i;
    }
}



/* Load color map/fmap files from standard directories */
void
load_color_maps (GList ** color_map_list)
{
    load_color_maps_from_dir (color_map_list, paths.maps_dir);
    load_color_maps_from_dir (color_map_list, LSYSEXP_MAPS);
}



/* Find and load all .map and .fmap files from "path" into the list
 * color_map_list */
void
load_color_maps_from_dir (GList ** color_map_list, char * path)
{
    GDir * dir;
    const char * name;
    
    if (! g_file_test (path, G_FILE_TEST_IS_DIR))
	return;
    if ((dir = g_dir_open (path, 0, NULL)) == NULL)
	return;

    while ((name = g_dir_read_name(dir)) != NULL)
    {
	char * name_lower = g_ascii_strdown (name, strlen(name));
 
	if ((ENDS_WITH (name_lower, ".map") || ENDS_WITH (name_lower, ".fmap"))
	    && ! g_list_find_custom (*color_map_list, name, color_map_find))
	{
	    map_type type = (ENDS_WITH (name_lower, ".fmap")) ? FMAP: MAP;
	    append_map_to_list (color_map_list, path, (char*)name, type);
	}
	g_free (name_lower);
    }
    g_dir_close (dir);
}



/* Add .map or .fmap file "file" to color_map_list */
void
append_map_to_list (GList ** color_map_list, char * path, char * file,
		    map_type type)
{
    char line [128];
    FILE * in;
    char * full_path;
    color_group * g;

    full_path = g_strdup_printf("%s/%s", path, file);
    
    if ((in = fopen (full_path, "r")) == NULL)
    {
	g_free (full_path);
	return;
    }
    g = (color_group *)g_malloc (sizeof(color_group));
    g->colors = NULL;
    g->name = g_strdup (file);
    g_strdelimit (g->name, ".", 0); /* get rid of extension */
    g->type = type;
    g->loop = 0;

    /* Read in each color */
    while (fgets (line, 127, in))
    {
	dcolor * c;
	int R, G, B;

	g_strstrip (line);
	
	/* skip blank lines and comments */
 	if (line[0] == 0 || line[0] == '#')
	    continue;
	else if (strstr (line, "loop"))
	{
	    g->loop = 1;
	    continue;
	}
	else if (strstr (line, "noloop"))
	{
	    g->loop = 0;
	    continue;
	}
	/* Text color string, man XParseColor for acceptable formats */
	else if (! g_ascii_isdigit(line[0]))
	{
            GdkColor clr;
            if (gdk_color_parse (line, &clr))
	    {
		c = (dcolor *)g_malloc(sizeof(dcolor));
		c->r = (double) clr.red;
		c->g = (double) clr.green;
		c->b = (double) clr.blue;
	    }
	    else continue;
	}
	/* RGB values */
	else
	{
	    sscanf(line, "%u %u %u", &R, &G, &B);   
	    c = (dcolor *)g_malloc(sizeof(dcolor));
	    c->r = ((double)R/(double)255 * (double)65535);
	    c->g = ((double)G/(double)255 * (double)65535);
	    c->b = ((double)B/(double)255 * (double)65535);
        }

	g->colors = g_list_append (g->colors, c);

    }

    g_free (full_path);
    fclose (in);

    (*color_map_list) = 
	g_list_insert_sorted ((*color_map_list), g,
			      (GCompareFunc) color_map_compare);
}



/* My own generalized garbage collector (not really) */
void
destroy_color_maps (GList ** color_map_list)
{
    int i, j;

    for (i = 0; i < g_list_length (*color_map_list); ++i)
    {
	color_group * g = (color_group *)g_list_nth_data(*color_map_list, i);
	for (j = 0; j < g_list_length (g->colors); ++j)
	{
	    dcolor * c = (dcolor *)g_list_nth_data(g->colors, j);
	    g_free (c);
	}
	g_list_free (g->colors);
	g_free (g->name);
	g_free (g);
    }
    g_list_free (*color_map_list);
}



/* Can someone implement lambda/anonymous functions for C?  PLEASE???????? */
gint
color_map_compare (gpointer a, gpointer b)
{
    return (g_ascii_strcasecmp (((color_group *)a)->name, ((color_group *)b)->name));
}


gint
color_map_find (gconstpointer list_data, gconstpointer name)
{
    int rc;
    char *s;
    color_group * g = (color_group *)list_data;

    /* Strip extension */
    s = g_strdup(name);
    g_strdelimit (s, ".", 0);
    rc = g_ascii_strcasecmp (g->name, s);
    g_free (s);
    return rc;
}

