/* colors.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Routines for choosing colors.
 *
 */

#include <math.h>    /* sqrt */
#include <string.h>  /* memset */

#include "colors.h"
#include "constants.h"
#include "util.h"

/*  Modifies (changes the red/green/blue values) in colors according to
 * "method" and "index"
 *
 *  At most ncolors will be modified, the actual number modified is returned
 *  (this may be less than ncolors)
 *
 *  "method" is either:
 *   COLOR_UNIFORM = A uniform (i.e., color spectrum) coloring
 *    COLOR_SMOOTH = A random smooth coloring
 *    COLOR_RANDOM = A completely random coloring (no fading)
 */
int
get_colors (GdkColor * colors, int ncolors, color_type method, int index)
{
    int requested = ncolors;

    if (method == COLOR_UNIFORM)
	make_uniform_colormap (colors, &requested);
    else if (method == COLOR_RANDOM)
	make_random_colormap (colors, &requested, TRUE);
    else if (method == COLOR_SMOOTH)
	make_smooth_colormap (colors, &requested);
   return requested;
}



/* Create a fading between ncolorsP colors in RGB space */
void
make_color_path_rgb (color_group * grp, GdkColor * colors, int *ncolorsP)
{
    int i, j, npoints, edge, index = 0;
    double r[MAXPOINTS], g[MAXPOINTS], b[MAXPOINTS];

    for (i = 0; 
	 i < *ncolorsP && i < g_list_length(grp->colors) && i < MAXCOLORS;
	 ++i)
    {
	dcolor * c = (dcolor *)g_list_nth_data (grp->colors, i);
	r[i] = (int)c->r;
	g[i] = (int)c->g;
	b[i] = (int)c->b;
    }
    
    /* fade backwards */
    if (grp->loop)
    {
	int size = i;
	for (i = 0; i < size; ++i)
        {
            r[i+size] = r[size-i-1];
            g[i+size] = g[size-i-1];
            b[i+size] = b[size-i-1];
        }
        i = size + size;
    }

    npoints = i - 1;
    edge = (*ncolorsP) / npoints;

    for (i = 0; i < npoints; ++i)
    {
	double ri, bi, gi;
	ri = (double)(r[i+1] - r[i]) / (double)edge;
	gi = (double)(g[i+1] - g[i]) / (double)edge;
	bi = (double)(b[i+1] - b[i]) / (double)edge;

	for (j = 0; j < edge; ++j)
	{
	    index = i * edge + j;
	    colors[index].red   = (r[i] + j*ri);
	    colors[index].green = (g[i] + j*gi);
	    colors[index].blue  = (b[i] + j*bi);
	}
    }
    *ncolorsP = index + 1;
}



/* ----------------------- Begin XScreenSaver functions ------------------
 * 
 *  xscreensaver, Copyright (c) 1991-2002 Jamie Zawinski <jwz@jwz.org>
 *
 * -----------------------------------------------------------------------
 */

void make_color_ramp (int h1, double s1, double v1,
		      int h2, double s2, double v2,
		      GdkColor *colors, int *ncolorsP,
		      gboolean closed_p)
{
    
  int i;
  int total_ncolors = *ncolorsP;
  int ncolors, wanted;
  double dh, ds, dv;		/* deltas */

  wanted = total_ncolors;
  if (closed_p)
    wanted = (wanted / 2) + 1;

  ncolors = total_ncolors;

  memset (colors, 0, (*ncolorsP) * sizeof(*colors));

  if (closed_p)
    ncolors = (ncolors / 2) + 1;

  /* Note: unlike other routines in this module, this function assumes that
     if h1 and h2 are more than 180 degrees apart, then the desired direction
     is always from h1 to h2 (rather than the shorter path.)  make_uniform
     depends on this.
   */
  dh = ((double)h2 - (double)h1) / ncolors;
  ds = (s2 - s1) / ncolors;
  dv = (v2 - v1) / ncolors;

  for (i = 0; i < ncolors; i++)
    {
      hsv_to_rgb ((int) (h1 + (i*dh)), (s1 + (i*ds)), (v1 + (i*dv)),
		  &colors[i].red, &colors[i].green, &colors[i].blue);
    }

  if (closed_p)
    for (i = ncolors; i < *ncolorsP; i++)
      colors[i] = colors[(*ncolorsP)-i];
}



void
make_color_loop (int h0, double s0, double v0,
		      int h1, double s1, double v1,
		      int h2, double s2, double v2,
		      GdkColor *colors, int *ncolorsP)
{
    int h[3];
    double s[3], v[3];
    h[0] = h0; h[1] = h1; h[2] = h2;
    s[0] = s0; s[1] = s1; s[2] = s2;
    v[0] = v0; v[1] = v1; v[2] = v2;
    make_color_path(3, h, s, v,
		    colors, ncolorsP);
}



void
make_smooth_colormap (GdkColor *colors, int *ncolorsP)
{
  int npoints;
  int ncolors = *ncolorsP;
  int i;
  int h[MAXPOINTS];
  double s[MAXPOINTS];
  double v[MAXPOINTS];
  double total_s = 0;
  double total_v = 0;

  if (*ncolorsP <= 0) return;

  {
    int n = g_random_int_range (0, 20);
    if      (n <= 5)  npoints = 2;	/* 30% of the time */
    else if (n <= 15) npoints = 3;	/* 50% of the time */
    else if (n <= 18) npoints = 4;	/* 15% of the time */
    else             npoints = 5;	/*  5% of the time */
  }

 REPICK_ALL_COLORS:
  for (i = 0; i < npoints; i++)
    {
    REPICK_THIS_COLOR:
	h[i] = g_random_int_range (0, 360); 
	s[i] = g_random_double_range (0.0, 1.0);
	v[i] = g_random_double_range (0.2, 1.0);

      /* Make sure that no two adjascent colors are *too* close together.
	 If they are, try again.
       */
      if (i > 0)
	{
	  int j = (i+1 == npoints) ? 0 : (i-1);
	  double hi = ((double) h[i]) / 360;
	  double hj = ((double) h[j]) / 360;
	  double dh = hj - hi;
	  double distance;
	  if (dh < 0) dh = -dh;
	  if (dh > 0.5) dh = 0.5 - (dh - 0.5);
	  distance = sqrt ((dh * dh) +
			   ((s[j] - s[i]) * (s[j] - s[i])) +
			   ((v[j] - v[i]) * (v[j] - v[i])));
	  if (distance < 0.2)
	    goto REPICK_THIS_COLOR;
	}
      total_s += s[i];
      total_v += v[i];
    }

  /* If the average saturation or intensity are too low, repick the colors,
     so that we don't end up with a black-and-white or too-dark map.
   */
  if (total_s / npoints < 0.2)
    goto REPICK_ALL_COLORS;
  if (total_v / npoints < 0.3)
    goto REPICK_ALL_COLORS;

  make_color_path (npoints, h, s, v, colors, &ncolors);

  *ncolorsP = ncolors;
}



void
make_uniform_colormap (GdkColor *colors, int *ncolorsP)
{
  int ncolors = *ncolorsP;

  double S = g_random_double_range (.66, 1.0); /* range 66%-100% */
  double V = g_random_double_range (.66, 1.0); /* range 66%-100% */

  if (*ncolorsP <= 0) return;

  make_color_ramp(0,   S, V,
		  359, S, V,
		  colors, &ncolors,
		  FALSE);
}



void
make_random_colormap (GdkColor *colors, int *ncolorsP,
			   gboolean bright_p)
{
  int ncolors = *ncolorsP;
  int i;

  if (*ncolorsP <= 0) return;

  for (i = 0; i < ncolors; i++)
    {
      if (bright_p)
	{
	  int H = g_random_int_range(0, 360);	       /* range 0-360    */
	  double S = g_random_double_range (.30, 1.0); /* range 30%-100% */
	  double V = g_random_double_range (.66, 1.0); /* range 66%-100% */
	  hsv_to_rgb (H, S, V,
		      &colors[i].red, &colors[i].green, &colors[i].blue);
	}
      else
	{
	    colors[i].red   = g_random_int_range(0, 0xFFFF);
	    colors[i].green = g_random_int_range(0, 0xFFFF);
	    colors[i].blue  = g_random_int_range(0, 0xFFFF);
	}
    }
}



void
make_color_path (int npoints, int *h, double *s, double *v,
		 GdkColor *colors, int *ncolorsP)
{
  int i, j, k;
  int total_ncolors = *ncolorsP;

  int ncolors[MAXPOINTS];  /* number of pixels per edge */
  double dh[MAXPOINTS];    /* distance between pixels, per edge (0 - 360.0) */
  double ds[MAXPOINTS];    /* distance between pixels, per edge (0 - 1.0) */
  double dv[MAXPOINTS];    /* distance between pixels, per edge (0 - 1.0) */

  if (npoints == 0)
    {
      *ncolorsP = 0;
      return;
    }
  else if (npoints == 2)	/* using make_color_ramp() will be faster */
    {
      make_color_ramp (h[0], s[0], v[0], h[1], s[1], v[1],
		       colors, ncolorsP,
		       TRUE);  /* closed_p */
      return;
    }
  else if (npoints >= MAXPOINTS)
    {
      npoints = MAXPOINTS-1;
    }

 AGAIN:

  {
    double DH[MAXPOINTS];	/* Distance between H values in the shortest
				   direction around the circle, that is, the
				   distance between 10 and 350 is 20.
				   (Range is 0 - 360.0.)
				*/
    double edge[MAXPOINTS];	/* lengths of edges in unit HSV space. */
    double ratio[MAXPOINTS];	/* proportions of the edges (total 1.0) */
    double circum = 0;
    double one_point_oh = 0;	/* (debug) */

    for (i = 0; i < npoints; i++)
      {
	int j = (i+1) % npoints;
	double d = ((double) (h[i] - h[j])) / 360;
	if (d < 0) d = -d;
	if (d > 0.5) d = 0.5 - (d - 0.5);
	DH[i] = d;
      }

    for (i = 0; i < npoints; i++)
      {
	int j = (i+1) % npoints;
	edge[i] = sqrt((DH[i] * DH[j]) +
		       ((s[j] - s[i]) * (s[j] - s[i])) +
		       ((v[j] - v[i]) * (v[j] - v[i])));
	circum += edge[i];
      }

#ifdef DEBUG
    fprintf(stderr, "\ncolors:");
    for (i=0; i < npoints; i++)
      fprintf(stderr, " (%d, %.3f, %.3f)", h[i], s[i], v[i]);
    fprintf(stderr, "\nlengths:");
    for (i=0; i < npoints; i++)
      fprintf(stderr, " %.3f", edge[i]);
#endif /* DEBUG */

    if (circum < 0.0001)
      goto FAIL;

    for (i = 0; i < npoints; i++)
      {
	ratio[i] = edge[i] / circum;
	one_point_oh += ratio[i];
      }

#ifdef DEBUG
    fprintf(stderr, "\nratios:");
    for (i=0; i < npoints; i++)
      fprintf(stderr, " %.3f", ratio[i]);
#endif /* DEBUG */

    /* space the colors evenly along the circumference -- that means that the
       number of pixels on a edge is proportional to the length of that edge
       (relative to the lengths of the other edges.)
     */
    for (i = 0; i < npoints; i++)
      ncolors[i] = total_ncolors * ratio[i];


#ifdef DEBUG
    fprintf(stderr, "\npixels:");
    for (i=0; i < npoints; i++)
      fprintf(stderr, " %d", ncolors[i]);
    fprintf(stderr, "  (%d)\n", total_ncolors);
#endif /* DEBUG */

    for (i = 0; i < npoints; i++)
      {
	int j = (i+1) % npoints;

	if (ncolors[i] > 0)
	  {
	    dh[i] = 360 * (DH[i] / ncolors[i]);
	    ds[i] = (s[j] - s[i]) / ncolors[i];
	    dv[i] = (v[j] - v[i]) / ncolors[i];
	  }
      }
  }

  memset (colors, 0, (*ncolorsP) * sizeof(*colors));

  k = 0;
  for (i = 0; i < npoints; i++)
    {
      int distance, direction;
      distance = h[(i+1) % npoints] - h[i];
      direction = (distance >= 0 ? -1 : 1);

      if (distance > 180)
	distance = 180 - (distance - 180);
      else if (distance < -180)
	distance = -(180 - ((-distance) - 180));
      else
	direction = -direction;

#ifdef DEBUG
      fprintf (stderr, "point %d: %3d %.2f %.2f\n",
	       i, h[i], s[i], v[i]);
      fprintf(stderr, "  h[i]=%d  dh[i]=%.2f  ncolors[i]=%d\n",
	      h[i], dh[i], ncolors[i]);
#endif /* DEBUG */
      for (j = 0; j < ncolors[i]; j++, k++)
	{
	  double hh = (h[i] + (j * dh[i] * direction));
	  if (hh < 0) hh += 360;
	  else if (hh > 360) hh -= 0;
	  hsv_to_rgb ((int)
		      hh,
		      (s[i] + (j * ds[i])),
		      (v[i] + (j * dv[i])),
		      &colors[k].red, &colors[k].green, &colors[k].blue);
#ifdef DEBUG
	  fprintf (stderr, "point %d+%d: %.2f %.2f %.2f  %04X %04X %04X\n",
		   i, j,
		   hh,
		   (s[i] + (j * ds[i])),
		   (v[i] + (j * dv[i])),
		   colors[k].red, colors[k].green, colors[k].blue);
#endif /* DEBUG */
	}
    }

  /* Floating-point round-off can make us decide to use fewer colors. */
  if (k < *ncolorsP)
    {
      *ncolorsP = k;
      if (k <= 0)
	return;
    }

  return;

 FAIL:
  /* we weren't able to allocate all the colors we wanted;
     decrease the requested number and try again.
   */
  total_ncolors = (total_ncolors > 170 ? total_ncolors - 20 :
		   total_ncolors > 100 ? total_ncolors - 10 :
		   total_ncolors >  75 ? total_ncolors -  5 :
		   total_ncolors >  25 ? total_ncolors -  3 :
		   total_ncolors >  10 ? total_ncolors -  2 :
		   total_ncolors >   2 ? total_ncolors -  1 :
		   0);
  *ncolorsP = total_ncolors;
  if (total_ncolors > 0)
    goto AGAIN;
}

