/* lsys.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Routines for loading/saving/retreiving builtin and user L-systems, and
 * loading/retrieving options.
 */

#include "lsys.h"
#include "builtin.h"
#include "util.h"
#include "config.h"
#include "constants.h"

#include <errno.h>  /* for write (fprintf(FILE, ...)) errors */

extern GList * groups;
extern path_info paths;

/* Load all builtin and user L-systems */
void
load_lsystems ()
{
    int i, j;
    groups = NULL;

    for (i = 0; i < countof(builtins); ++i)
    {
	lsys_group * group;
	/* add current group to groups list */
	group = (lsys_group *) g_malloc(sizeof(lsys_group));
	group->group_name = g_strdup (builtins[i].group_name);
	group->lsys_head = NULL;

	/* add each lsys in current group to head of current group list */
	for (j = 0; j < builtins[i].size; ++j)
	{
	    lsys * lsys_temp = (lsys *)g_malloc(sizeof(lsys));

	    /* copy builtin lsys to list */
	    lsys_temp->level = builtins[i].l[j].level;
	    lsys_temp->da = builtins[i].l[j].da;
   	    lsys_temp->a0 = builtins[i].l[j].a0;
	    lsys_temp->ds = builtins[i].l[j].ds;
	    lsys_temp->axiom = g_strdup(builtins[i].l[j].axiom);
	    lsys_temp->rule = g_strdup(builtins[i].l[j].rule);
	    lsys_temp->descrip = g_strdup(builtins[i].l[j].descrip);

	    group->lsys_head = g_list_insert_sorted (group->lsys_head,
						      lsys_temp,
						      (GCompareFunc) lsys_compare);
	}
	groups = g_list_insert_sorted (groups, group, (GCompareFunc) group_compare);
    }
    load_user_lsystems();
}



/* Load custom user L-systems */
void
load_user_lsystems()
{
    FILE * in;
    char * group_name;
    char line[256];
    lsys current;

#define VALID_KEY(k) (!(   g_ascii_strcasecmp(k, "a0") \
			&& g_ascii_strcasecmp(k, "da") \
                        && g_ascii_strcasecmp(k, "ds") \
			&& g_ascii_strcasecmp(k, "axiom") \
                        && g_ascii_strcasecmp(k, "rules") \
                        && g_ascii_strcasecmp(k, "level") \
                        && g_ascii_strcasecmp(k, "group") \
                        && g_ascii_strcasecmp(k, "name")))

    /* Establish defaults */
    current.da = 60.0;
    current.ds = 0.5;
    current.a0 = 90.0;
    current.level = 2;
    group_name = current.descrip = current.rule = current.axiom = NULL;


    /* File doesn't exist, nothing to load */
    if ((in = fopen (paths.custom_file, "r")) == NULL)
    {
	return;
    }

    /* Read in each line */
    while (fgets (line, 255, in))
    {
	int nfields;
	char ** fields;
	g_strchomp (line);
	fields = g_strsplit (line, ":", 2);

	for (nfields = 0; nfields < 3 && fields[nfields]; ++nfields)
	{
	    g_strstrip(fields[nfields]);
	    if (! fields[nfields][0] || (nfields + 1) > 2)
		break;
	}

	/* } */
	if (nfields == 1 && fields[0][0] == '}')
	{
	    /* These four fields must be specified for us to add lsys */
 	    if (group_name && current.rule && current.axiom && current.descrip)
	    {
		add_lsys (group_name, current.descrip, current.axiom,
			     current.rule, current.a0, current.da,
			     current.ds, current.level);
	    }

            g_free (current.descrip);
            g_free (current.axiom);
            g_free (group_name);

	    /* Re-establish defaults */
	    current.da = 60.0;
	    current.ds = 0.5;
	    current.a0 = 90.0;
	    current.level = 2;
	    group_name = current.rule = current.axiom = current.descrip = NULL;
	}

	/* fields[0]: fields[1] */
	else if (nfields == 2 && VALID_KEY(fields[0]))
	{
	    /* numeric values */
	    if (g_ascii_strcasecmp (fields[0], "a0") == 0)
		current.a0 = g_ascii_strtod(fields[1], NULL);
	    else if (g_ascii_strcasecmp (fields[0], "da") == 0)
		current.da = g_ascii_strtod(fields[1], NULL);
	    else if (g_ascii_strcasecmp (fields[0], "ds") == 0)
		current.ds = g_strtod(fields[1], NULL);
	    else if (g_ascii_strcasecmp (fields[0], "level") == 0)
		current.level = (int)g_strtod(fields[1], NULL);
	    
	    /* string values */
	    else if (g_ascii_strcasecmp (fields[0], "axiom") == 0)
		current.axiom = g_strdup (fields[1]);
	    else if (g_ascii_strcasecmp (fields[0], "rules") == 0)
	    {
		/* append rule */
		if (current.rule != NULL)
		{
		    char * save = current.rule;
		    current.rule = g_strjoin (";", current.rule, fields[1], NULL);
		    g_free (save);
		}
		else
		    current.rule = g_strdup (fields[1]);
	    }
	    else if (g_ascii_strcasecmp (fields[0], "group") == 0)
		group_name = g_strdup (fields[1]);
	    else /* descrip */
 		current.descrip = g_strdup(fields[1]);
	}
	g_strfreev (fields);
    }
    fclose (in);
}



/* Save the L-system described by the arguments and add it to the current list 
 * of L-systems (and save it in paths.custom_file) */
int
save_lsys (char * group_name, char * descrip, char * axiom,
	   char * rule, double a0, double da, double ds, int level)
{
    int i;
    FILE * out;
    char ** rules;

    /* open for create/append */
    out = fopen (paths.custom_file, "a");

    /* Couldn't write the file, return error */
    if (out == NULL)
    {
	return errno;
    }
    
    /* Output  L-system */
    fprintf (out, "{\n");
    fprintf (out, "  group: %s\n", group_name);
    fprintf (out, "   name: %s\n", descrip);
    fprintf (out, "  axiom: %s\n", axiom);

    /* split up rules, for readability */
    rules = g_strsplit (rule, "\n", 0);
    for (i = 0; rules[i]; ++i)
    {
	g_strstrip (rules[i]);
	if (! rules[i][0]) continue;
	fprintf (out, "  rules: %s\n", rules[i]); 
    }
    g_strfreev (rules);

    fprintf (out, "     a0: %f\n", a0);
    fprintf (out, "     da: %f\n", da);
    fprintf (out, "     ds: %f\n", ds);
    fprintf (out, "  level: %d\n", level);
    fprintf (out, "}\n\n");

    fclose (out);

    /* Add L-system to current list of L-systems */
    add_lsys (group_name, descrip, axiom, rule, a0, da, ds, level);

    return 0;
}



/* Add the L-system described by the arguments to the current list of L-systems
 */
void
add_lsys (char * group_name, char * descrip, char * axiom, char * rules,
             double a0, double da, double ds, int level)
{
    int i, j;
    lsys_group * group_temp;
    lsys * lsys_temp;

    for (i = 0; i < g_list_length(groups); ++i)
    {
	lsys_group * g = (lsys_group *)g_list_nth_data(groups, i);
	if (g_ascii_strcasecmp (group_name, g->group_name) == 0)
	{
	    /* See if lsys already exists in this group */
 	    for (j = 0; j < g_list_length(g->lsys_head); ++j)
	    {
		lsys * frac = (lsys *)g_list_nth_data(g->lsys_head, j);

		/* replace existing lsys */
		if (g_ascii_strcasecmp (descrip, frac->descrip) == 0)
		{
		    g_free (frac->axiom);
		    g_free (frac->rule);

		    frac->axiom = g_strdup (axiom);
		    frac->rule  = g_strdup (rules);
		    frac->a0 = a0;
		    frac->da = da;
		    frac->ds = ds;
		    frac->level = level;
		    return;
		}
	    }
	    /* add lsys to group */
	    lsys_temp = (lsys *)g_malloc(sizeof(lsys));
	    lsys_temp->axiom = g_strdup (axiom);
	    lsys_temp->rule  = g_strdup (rules);
	    lsys_temp->descrip = g_strdup (descrip);
	    lsys_temp->a0 = a0;
	    lsys_temp->da = da;
	    lsys_temp->ds = ds;
	    lsys_temp->level = level;
	    g->lsys_head = g_list_insert_sorted (g->lsys_head,
						 lsys_temp,
						 (GCompareFunc)lsys_compare);
	    return;
	}
    }

    /* group and lsys not found, create new group and add lsys to it */
    group_temp = (lsys_group *)g_malloc(sizeof(lsys_group));
    group_temp->lsys_head = NULL;
    group_temp->group_name = g_strdup (group_name);
    lsys_temp = (lsys *)g_malloc(sizeof(lsys));
    lsys_temp->axiom = g_strdup (axiom);
    lsys_temp->rule  = g_strdup (rules);
    lsys_temp->descrip = g_strdup (descrip);
    lsys_temp->a0 = a0;
    lsys_temp->da = da;
    lsys_temp->ds = ds;
    lsys_temp->level = level;
    group_temp->lsys_head = g_list_insert_sorted (group_temp->lsys_head,
						  lsys_temp,
						  (GCompareFunc)lsys_compare);
    groups = g_list_insert_sorted (groups, group_temp,
				   (GCompareFunc) group_compare);
}



/* Returns 1 if the L-system "descrip" already exists in the group "group_name"
 */
int
lsys_exists (char * group_name, char * descrip)
{
    int i, j;
    for (i = 0; i < g_list_length(groups); ++i)
    {
	lsys_group * g = (lsys_group *)g_list_nth_data(groups, i);
	if (g_ascii_strcasecmp (g->group_name, group_name) == 0)
	{
	    for (j = 0; j < g_list_length(g->lsys_head); ++j)
	    {
		lsys * frac = (lsys *)g_list_nth_data(g->lsys_head, j);
		if (g_ascii_strcasecmp (frac->descrip, descrip) == 0)
		    return 1;
	    }
	}
    }
    return 0;
}



/* Delete memory occupied by L-systems list */
void
destroy_lsystems()
{
    int i, j;
    for (i = 0; i < g_list_length (groups); ++i)
    {
	lsys_group * g = (lsys_group *)g_list_nth_data(groups, i);
	GList * lsystems = g->lsys_head;
	for (j = 0; j < g_list_length(lsystems); ++j)
	{
	    lsys * lsys_tmp = (lsys *) g_list_nth_data (lsystems, j);
	    g_free (lsys_tmp->axiom);
	    g_free (lsys_tmp->rule);
	    g_free (lsys_tmp->descrip);
	    g_free (lsys_tmp);
	}
	g_free (g->group_name);
	g_list_free (lsystems);
	g_free (g);
    }
    g_list_free (groups);
}



/* Return all the relevant information about the L-system "lsys" in 
 * group "group"
 */
int
get_lsys_info (char * group_name, char * lsys_name,
	       char ** axiom, char ** rules, 
	       double * da, double * a0, int * level, double * ds)
{
    int i, j;

    for (i = 0; i < g_list_length(groups); ++i)
    {
	lsys_group * g = (lsys_group *)g_list_nth_data(groups, i);
	if (g_ascii_strcasecmp(g->group_name, group_name) == 0)
	{
	    for (j = 0; j < g_list_length(g->lsys_head); ++j)
	    {
		lsys * frac = (lsys *)g_list_nth_data(g->lsys_head, j);
		if (g_ascii_strcasecmp (frac->descrip, lsys_name) == 0)
		{
		    (*axiom) = frac->axiom;
		    (*rules) = g_strdelimit(g_strdup(frac->rule),
					    ";", '\n');
		    (*da) = frac->da;
		    (*a0) = frac->a0;
		    (*level) = frac->level;
		    (*ds) = frac->ds;
		    return 0;
		}
	    }
	}
    }
    return -1;
}



/* Return a GList of all the current group names */
GList*
get_group_names_list ()
{
    GList * items = NULL;
    int i;

    for (i = 0; i < g_list_length(groups); ++i)
    {
	char * name = ((lsys_group *)g_list_nth_data(groups, i))->group_name;
	items = g_list_append (items, name);
    }
    
    return items;
}



/* Return a GList of all the L-systems in group "group_name" */
GList*
get_lsys_names_list (char * group_name)
{
    GList * items = NULL;

    int i, j;

    for (i = 0; i < g_list_length(groups); ++i)
    {
	lsys_group * g = (lsys_group *)g_list_nth_data(groups, i);
	if (g_ascii_strcasecmp(g->group_name, group_name) == 0)
	{
	    for (j = 0; j < g_list_length(g->lsys_head); ++j)
	    {
		char * name  = ((lsys *)g_list_nth_data(g->lsys_head, j))->descrip;
		items = g_list_append (items, name);
	    }
	}
    }

    return items;
}



/* Return the number of L-systems in group "group_name" */
int
get_lsys_count (char * group_name)
{
    int i, n = -1;

    for (i = 0; i < g_list_length(groups); ++i)
    {
	lsys_group * g = (lsys_group *)g_list_nth_data(groups, i);
	if (g_ascii_strcasecmp(g->group_name, group_name) == 0)
	{
	    n = g_list_length (g->lsys_head);
	    break;
	}
    }

    return n;
}



/* Return a GtkMenu with all the groups */
GtkWidget*
get_group_names_menu ()
{
    return menu_from_list (get_group_names_list());
}



/* Return a GtkMenu with all the L-systems in group "group_name" */
GtkWidget*
get_lsys_names_menu (char * group_name)
{
    return menu_from_list (get_lsys_names_list(group_name));
}



/* For sorting group names */
gint
group_compare (gpointer a, gpointer b)
{
    lsys_group *ag = (lsys_group *)a;
    lsys_group *bg = (lsys_group *)b;

    return (g_ascii_strcasecmp (ag->group_name, bg->group_name));
}



/* For sorting L-system names */
gint
lsys_compare (gpointer a, gpointer b)
{
    return (g_ascii_strcasecmp (((lsys *)a)->descrip, ((lsys *)b)->descrip));
}



int
save_user_options (opts * defaults)
{
    FILE * out;

    if ((out = fopen (paths.options_file, "w")) == NULL)
	return errno;

    fprintf(out, "%u %u %u %hd %hd %hd %u\n%s\n",
	    defaults->width, defaults->height, defaults->ncolors,
	    defaults->bgcolor.red, defaults->bgcolor.green,
	    defaults->bgcolor.blue, 
	    defaults->center_window,
	    defaults->save_dir);

    fclose (out);
    return 0;
}

void
load_user_options (opts * defaults)
{
    FILE * in;
    defaults->width = 400;
    defaults->height = 300;
    defaults->ncolors = DEFAULT_COLORS;
    defaults->bgcolor.red = defaults->bgcolor.green = defaults->bgcolor.blue = 0;
    defaults->save_dir = NULL;
    defaults->center_window = 1;

    if ((in = fopen (paths.options_file, "r")) != NULL)
    {
	char buf[256];
	if (fgets (buf, 255, in))
	{
	    /* width, height, ncolors, bgcolor.(red,green,blue), center window */
	    sscanf (buf, "%u %u %u %hd %hd %hd %u", 
		    &(defaults->width),
		    &(defaults->height),
		    &(defaults->ncolors),
		    &(defaults->bgcolor.red),
		    &(defaults->bgcolor.green),
		    &(defaults->bgcolor.blue),
		    &(defaults->center_window));

	    /* sanity checks, in case users edit the options file by hand which
	     * they aren't supposed to do... */
 	    defaults->width = MIN(gdk_screen_width(), defaults->width);
	    defaults->height = MIN(gdk_screen_height(), defaults->height);
	    if (defaults->width < 25) defaults->width = 25;
	    if (defaults->height < 25) defaults->height = 25;
	    defaults->ncolors =
		(defaults->ncolors < 32) ? MINCOLORS :
		(defaults->ncolors > 1024) ? MAXCOLORS :
		defaults->ncolors;
	}
	if (fgets (buf, 255, in))
	{
	    g_strstrip(buf);
	    if (g_file_test(buf, G_FILE_TEST_EXISTS)
		&& g_file_test(buf, G_FILE_TEST_IS_DIR))
		defaults->save_dir = g_strdup (buf);
	}
    }

    if (in) fclose (in);

    /* Make save directory CWD, with / (if !windows) appended on the end */
    if (defaults->save_dir == NULL)
    {
	char * sep = "/";
	char *d = g_get_current_dir();
#ifdef WINDOWS
	sep = "";  /* glib doesn't think c:\foo\bar\ is a valid dir... */
#endif
	defaults->save_dir = g_strdup_printf("%s%s", d, sep);
	g_free (d);
    }
}

