/* lsys_draw.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Routines used to draw the L-systems, parse the rules, and determine
 * dimensions
 *
 */

#include <math.h>      /* sin, cos */
#include <string.h>    /* strchr, strlen */

#include "dialog_ui.h"
#include "lsys_draw.h"
#include "util.h"

extern GRand *RNG;
extern char stubs [];

/* Parses a list of rules given in "rule_string", which contains a
 * list of grammar productions, each separated by a \n or ; character.
 * 
 * For example:   rule_string = "f=f++g++f;g=f[-g+]fh;h=gg"
 *
 * rules[letter - 'a'] will then contain the rule for letter 'letter' for all
 * letters, a-z.
 *
 * In our prior example: rules['f' - 'a']="f++g++f"
 *
 * If any syntax errors are detected, it will display an error message and
 * return 1, otherwise it will return 0.
 *
 * Note: this function really should be cleaned up using the glib string 
 * functions.
 */
int
parse_rules (char *rule_string, lsys_window * win)
{
  char * start, * end, letter;
  int i;
  int equal;
  int last = 0;
  
  start = rule_string;
  for (i = 0; i < 'z' - 'a' + 1; ++i)
      win->rules[i] = &stubs[2 * i];

  /* Convert the whole thing to lower case */
  for (i = 0; start[i]; ++i)
      start[i] = g_ascii_tolower(start[i]);

  /* Determine the start and ending spot of each rule separated by ';', and
   * add it to rules[letter - 'a'] */  
  while (! last)
    {
      /* skip leading whitespace */
      for (i = 0; start[i] && g_ascii_isspace(start[i]); start++, i++);
      end = strchr(start, ';');
      if (! end)
	{
	  last = 1;
	  end = &start[strlen(start)];
	}

      /* change the ';' to 0 */
      *end = 0;

      /******* Syntax Checking *******/

      /* Has to contain at least "letter=" */
      if (! (start[0] && start[1]))
	{
	  char * err = g_strdup_printf("Syntax error at beginning of \"%s\"", start);
	  ok_dialog ("Error: Rule", err, GTK_WINDOW(win->main_win->win));
	  g_free (err);
	  return 1;
	}
      /* Make sure it's a letter [a-z] */
      letter = g_ascii_tolower (start[0]);
      if (! g_ascii_islower(letter))
	{
	  char * err = g_strdup_printf("Illegal grammar symbol %c, must be a-z", letter);
	  ok_dialog ("Error: Rule", err, GTK_WINDOW(win->main_win->win));
	  g_free (err);
	  return 1;
	}

      /* Needs to be:  letter=...., if there's no '=', then there's an error */
      equal = (int)strchr (start, '=');
      if (! equal)
	{
          char * err = g_strdup_printf("Syntax error: missing '=' in %s", start);
	  ok_dialog ("Error: Rule", err, GTK_WINDOW(win->main_win->win));
	  g_free (err);
	  return 1;
	}

      /* Passed the syntax checking */
      equal = (1 + equal - (int)start);
      win->rules[letter - 'a'] = &start[equal];
      start = 1 + end;
    }
  return 0;
}



/* Retreive all information necessary about this L-system from the main window,
 * and fill up "lsys_window * win" appropriately.  On error, returns non-zero.
 */
int
get_current_lsys (lsys_window * win)
{
    win->rule_string = g_strdelimit(get_textview_text (win->main_win->rules),
				      "\n", ';');

    /* Parse the rules, if something fails, return error */
    if (parse_rules (win->rule_string, win))
    {
	g_free (win->rule_string);
	win->rule_string = NULL;
	return 1;
    }

    /* Parse axiom */
    win->axiom = g_strdup(gtk_entry_get_text (GTK_ENTRY(win->main_win->axiom))); 
    if (!win->axiom[0])
    {
	ok_dialog ("Error: Axiom", "Axiom cannot be blank", GTK_WINDOW(win->main_win->win));
	g_free (win->axiom);
	g_free (win->rule_string);
	win->axiom = win->rule_string = NULL;
	return 1;
    }
    
    /* Get a0, da, ds, and level; convert degrees to radians */
    win->a0 = RADIANS(gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->main_win->s_a0)));
    win->da = RADIANS(gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->main_win->s_da)));
    win->ds = gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->main_win->s_ds));
    win->level = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->main_win->s_level));

    return 0;
}



/* Paint the L-system fractal on the screen */
void
draw_lsys (lsys_window * win)
{
    double noise;
    int border, i;

    win->level   = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_level));
    border  = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_border));
    noise   = gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_noise));

    /* Create pixmap and gc */
    if (win->pixmap == NULL)
    {
	win->pixmap = gdk_pixmap_new (win->drawing_area->window,
				      win->drawing_area->allocation.width,
				      win->drawing_area->allocation.height,
				      -1);

	win->gc = gdk_gc_new (win->drawing_area->window);
    }

    /* If border or noise level changes, that invalidates the information in 
     * each level */
    if (border != win->border || (ABS(noise - win->noise) > .02))
    {
	for (i = 0; i < MAXLEVELSIZE; ++i)
	    win->levels[i].used = 0;
    }

    win->border = border;
    win->noise = noise;

    /* We use this to ensure the scale of the image is correct */
    win->dim = MIN(win->width, win->height) - 2*win->border;

    /* Set pixmap to background color */
    gdk_gc_set_rgb_fg_color (win->gc, &(win->color.bgcolor));
    gdk_draw_rectangle (win->pixmap, win->gc,
			TRUE,
			0, 0,
			win->width,
			win->height);

    /* Find the size of the L-system so we can scale it appropriately */
    get_lsys_size (win, win->level);

    /* Need to seed the RNG so we get the same string of random numbers back */
    g_rand_set_seed (RNG, win->levels[win->level].seed);

    /* Draw the actual L-system */
    draw_rule (win, win->axiom, win->level, 1, NULL, win->level);

    /* Transfer L-system to actual window */
    gdk_draw_drawable (win->drawing_area->window,
		     win->drawing_area->style->black_gc,
		     win->pixmap,
		     0, 0, 0, 0, win->width, win->height);
}



void
get_lsys_size (lsys_window * win, int depth)
{
    guint32 line_count = 0;
    win->pos.x = win->pos.y = 0.0;
    win->pos.a = win->pos.explicit_a = win->a0;
    win->pos.s = 1;
    win->pos.reverse = 0;

    /* Haven't cached it, find out the window size, and save it */
    if (! win->levels[depth].used) 
    {
	double scale;
	double dx, dy;
      
	/* Only an approximation ... */
	win->levels[depth].maxx = win->levels[depth].maxy = -Inf;
	win->levels[depth].minx = win->levels[depth].miny = Inf;
	line_count = 0;
	win->levels[depth].used = 1;

	win->levels[depth].seed = glib_time() * depth + 1;
	g_rand_set_seed (RNG, win->levels[depth].seed);

	/* fill up win->levels[depth].(minx, maxx, miny, maxy) */
 	draw_rule (win, win->axiom, depth, 0, &line_count, depth);

	/* determine scale amount and offsets so it all fits on the window */
	dx = ( win->levels[depth].maxx -  win->levels[depth].minx);
	dy = ( win->levels[depth].maxy -  win->levels[depth].miny);
	scale = MIN(win->dim/dx,win->dim/dy);
	win->levels[depth].xo = (win->width  - (scale * dx))/2.0;
	win->levels[depth].yo = (win->height - (scale * dy))/2.0;
	win->levels[depth].scale = scale;

	/* still possible for line_count to be zero (for those late-bloomers) */
	win->levels[depth].line_count = line_count ? line_count : 1;
    }

    /* Set these back to initial conditions */
    win->pos.x = win->pos.y = 0.0;
    win->pos.a = win->pos.explicit_a = win->a0;
    win->pos.s = 1;
    win->pos.reverse = 0;
    win->color.color_index = 0.0;

    /* Calculate the increment used for coloring, line_count should always be
     * the same, but the # of colors may change */
    win->levels[depth].color_increment = (double)(win->color.ncolors) / (double)win->levels[depth].line_count;
}



/* Draw the actual L-system (i.e., this is the L-system interpreter)
 *
 * win: Information about the L-system to draw (see interface.h)
 * rule: The rule to interpret
 * depth: The current recursion depth (starts high decrements eventually to 0)
 * draw: If true, pain the L-system, otherwise just calculate the dimensions.
 * line_count: Used to determine the number of lines (to do smooth coloring)
 * max_depth: The value of "depth" when this function is first called
 *
 * Note 1: There's some copy/paste going on in here, and this is a rather long
 * function, but since it gets called so often it's faster to do this than
 * splitting it up into other functions (that's my excuse anyway).
 *
 * Note 2: This function is heavily inspired from the one given by Gary 
 * William Flake in the accompanying source code to his (excellent) book 
 * The Computational Beauty of Nature, which you can find at 
 *    http://mitpress.mit.edu/books/FLAOH/cbnhtml/
 */
void
draw_rule (lsys_window * win, char * rule, int depth, int draw,
	   guint32 * line_count, int max_depth)
{
    int i, turn_num;
    int sreverse;
    double sx, sy, sa, ss;    /* save coords, angle, scale */
    double ax, ay, bx, by;    /* temporary points */
    double da, ds;            /* turn angle & scaling factor */
    double sexplicit;

    level_info *level = &(win->levels[max_depth]);

    da = win->da;
    ds = win->ds;

    turn_num = 0;  /* keeps track how many times to turn */

    if (rule == NULL) return;

    for (i = 0; rule[i]; ++i)
    {
	char c = g_ascii_tolower(rule[i]);
      
	/* Flip meaning of +, -, \, and / */
	if (win->pos.reverse)
	{
	     c = (c == '-') ? '+' :
		 (c == '+') ? '-' :
		 (c == '/') ? '\\' :
		 (c == '\\') ? '/':
		 c;
	}

	/* letter or % */
	if (g_ascii_islower (c) || c == '%')
	{
	    /* recursively expand rules[c - 'a'] if we haven't reached final
	     * depth */
	    if (depth > 0 && c != '%')
	    {
		ss = win->pos.s;
		win->pos.s *= ds;
		draw_rule (win, win->rules[c - 'a'], depth - 1, draw, line_count, max_depth);
		win->pos.s = ss;
	    }

	    /* A draw or move command */
	    else if (c == 'f' || c == 'g' || c == '%'
		     || c == 'd' || c == 'm')
	    {
		/* Find out which angle to use */
		double ang = ((c == 'm' || c == 'd')) ? win->pos.explicit_a 
		    : win->pos.a;

		sx = win->pos.x + 
		    win->pos.s *
		    sin (ang + win->noise*g_rand_double_range(RNG, -1.0, 1.0));
		
		sy = win->pos.y + 
		    win->pos.s *
		    cos (ang + win->noise*g_rand_double_range(RNG, -1.0, 1.0));
		
		/* 'f', 'd', and '%' == draw commands */
		if (c == 'f' || c == '%'
		    || c == 'd')
		{ 
		    /* This is for the draw mode (lines/points/random) */
 		    int r = g_rand_int(RNG) & 1;
		    ax = win->dim * win->pos.x  + 0.5;
		    ay = win->dim * win->pos.y  + 0.5;
		    bx = win->dim * sx + 0.5;
		    by = win->dim * sy + 0.5;

		    /* Don't draw, just calculate window size */
		    if (! draw)
		    {
			level->maxx = MAX(level->maxx, MAX(ax, bx));
			level->maxy = MAX(level->maxy, MAX(win->dim-ay, win->dim-by));
			level->minx = MIN(level->minx, MIN(ax, bx));
			level->miny = MIN(level->miny, MIN(win->dim-ay, win->dim-by));
			if (line_count) (*line_count)++;
		    }		  

		    /* Convert to screen coords, and draw */
		    else
		    {
			double x1, y1, x2, y2;
			
#define to_x(x) (((x)-level->minx) * level->scale + level->xo)
#define to_y(y) (((y)-level->miny) * level->scale + level->yo)

			x1 = to_x(ax);
			y1 = to_y(win->dim - ay);
			x2 = to_x(bx);
			y2 = to_y(win->dim - by);

			/* Set current color and update color index */
			gdk_gc_set_rgb_fg_color (win->gc, 
		          &(win->color.colors[(int)(win->color.color_index)]));
			win->color.color_index += level->color_increment;
			if ((int)(win->color.color_index) >= win->color.ncolors)
			    win->color.color_index = 0.0;

			/* Draw using lines */
			if (win->mode == MODE_LINES
			    || (win->mode == MODE_RANDOM && r))
			{ 
			    gdk_draw_line (win->pixmap, win->gc,
					   x1, y1, x2, y2);
			}

			/* Draw using points */
			if (win->mode == MODE_POINTS
			    || (win->mode == MODE_RANDOM && !r))
			{ 
			    gdk_draw_point (win->pixmap, win->gc, x1, y1);
			    gdk_draw_point (win->pixmap, win->gc, x2, y2);
			}	
		    }
		}
		/* save new position */
		win->pos.x = sx;
		win->pos.y = sy;
	    }
	}

	/* it's a digit, add it to digit total for +/- turn later */
	else if (c >= '0' && c <= '9')
	{
	    turn_num = turn_num * 10 + (c - '0');
	}

	/* reverse meaning of + and - */
 	else if (c == '!')
	{
	    win->pos.reverse = ! win->pos.reverse;
	}

	/* Turn 180 degrees (or PI radians) */
	else if (c == '|')
	{
		win->pos.a += G_PI;
	}
 
	/* Adjust the explicit angle */
	else if (c == '\\' || c == '/')
	{
	    int j = 0;
	    double new_angle;
	    char new_num[32], direction = c;
	    i++;
	    while (rule[i]
		   && ((rule[i] >= '0' && rule[i] <= '9') || rule[i] == '.'))
		new_num[j++] = rule[i++];
	    new_num[j] = 0;
	    i--;
	    
	    new_angle = RADIANS(g_ascii_strtod(new_num, NULL));
	    win->pos.explicit_a += (direction == '/') ? new_angle : (-new_angle);
	}

	/* Multiply current line length by what follows @, I means inverse,
	* Q means square root */
	else if (c == '@')
	{
	    int j = 0, root = 0, inverse = 0;
	    char new_num[32];
	    double result;

	    /* Note: this is for compatability with fractint, where @IQ5 and 
	     * @QI5 are treated as equal (square root first, then inverse), so
	     * it's basically impossible to take the inverse of a square
	     * root, yay! */

	    i++;
	    switch (g_ascii_tolower(rule[i]))
	    {
	    case 'q':
		root = 1;
		i++;
		break;
	    case 'i':
		inverse = 1;
		i++;
		break;
	    }

	    switch (g_ascii_tolower(rule[i]))
	    {
	    case 'q':
		root = 1;
		i++;
		break;
	    case 'i':
		inverse = 1;
		i++;
	 	break;
	    }
	    
	    /* Scan floating point number */
	    while (rule[i] && j < 31 && 
		   ((rule[i] >= '0' && rule[i] <= '9') || rule[i] == '.'))
	    {
		new_num[j++] = rule[i++]; 
	    }
	    i--;
	    new_num[j] = 0;

	    result = g_ascii_strtod(new_num, NULL);
	    if (root)
		result = sqrt(result);
	    if (inverse)
		result = 1.0 / result;
	    win->pos.s *= result;
	}

	/* change direction "turn_num" times to the left (-) or right (+) */
	else if (c == '+' || c == '-')
	{
	    if (! turn_num) turn_num = 1;
	    if (c == '+')
	    {
		win->pos.a += turn_num * win->da;
		if (win->pos.a == 2.0*G_PI)
		    win->pos.a = 0;
	    }
	    else
	    {
		win->pos.a -= turn_num * win->da;
		if (win->pos.a == 0.0)
		    win->pos.a = 2.0*G_PI;
	    }
	    turn_num = 0;
	}

	/* save our current state, recursively call what's after '[', 
	 * and restore the state later */
	else if (c == '[')
	{
	    /* save state */
	    sx = win->pos.x; sy = win->pos.y;
	    sa = win->pos.a; ss = win->pos.s;
	    sreverse = win->pos.reverse;
	    sexplicit = win->pos.explicit_a;

	    /* Call what's after '[' */
 	    draw_rule (win, rule + i + 1, depth, draw, line_count, max_depth);

	    /* restore state */
	    win->pos.x = sx;
	    win->pos.y = sy; 
	    win->pos.a = sa;
	    win->pos.s = ss;
	    win->pos.reverse = sreverse;
	    win->pos.explicit_a = sexplicit;
	    turn_num = 1;

	    /* scan forward until we get a properly nested [] pair */
	    do
	    {
		i++;
		turn_num = turn_num + ((rule[i] == '[') ? 1 : (rule[i] == ']') ? -1 : 0);
	    }
	    while (turn_num != 0 && rule[i+1]);
	}

	/* already took care of part above, return */
	else if (c == ']')
	{
	    return;
	}
    } 
}
