/* lsyswin_cb.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Callback functions for L-system windows
 * 
 */

#include <string.h>

#include "lsyswin_cb.h"
#include "colors.h"
#include "constants.h"
#include "lsys_draw.h"
#include "color_map.h"
#include "dialog_ui.h"
#include "util.h"
#include "config.h"

extern struct _opts options;
extern GList * color_map_list;

/* -----------------------------------------------------------------------
 * 
 *  L-system window callbacks
 *
 * -----------------------------------------------------------------------
 */

void
on_lsys_win_destroy_event (GtkWidget *w, lsys_window * win)
{
    if (win->pixmap) g_object_unref (win->pixmap);
    if (win->rule_string) g_free (win->rule_string);
    if (win->axiom) g_free (win->axiom);
    if (win->tooltips) g_object_unref(win->tooltips);
    g_free (win->color.colors);
    g_free (win);
}



gboolean
on_lsys_win_delete_event(GtkWidget *w, GdkEvent *event, 
			 gpointer user_data)
{
      gtk_widget_destroy (w);
      return FALSE;
}



void
on_lsys_win_draw (GtkButton *button, lsys_window *win)
{
      draw_lsys (win);
}



void
on_lsys_win_draw_previous (GtkButton *button, lsys_window *win)
{
    int new_level;
    new_level = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_level));

    if (new_level > 0)
    {
	new_level--;
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(win->s_level), new_level);
	draw_lsys (win);
    }
}



void
on_lsys_win_draw_next (GtkButton *button, lsys_window *win)
{    
    int new_level;
    new_level = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_level));

    if (new_level < MAXLEVELSIZE)
    {
	new_level++;
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(win->s_level), new_level);
	draw_lsys (win);
    }
}



gint
on_lsys_win_drawing_expose (GtkWidget * widget, GdkEventExpose * event, 
			    lsys_window * win)
{
    gdk_draw_drawable (win->drawing_area->window,
		       win->gc,
		       win->pixmap,
		       event->area.x, event->area.y,
		       event->area.x, event->area.y,
		       event->area.width, event->area.height);
    return FALSE;
}



void
on_lsys_win_change_colors_map (GtkMenuItem * menu_item, lsys_window *win)
{
    char * name;

    if (!GTK_CHECK_MENU_ITEM(menu_item)->active)
	return;
    
    name = (char *) gtk_label_get_text (GTK_LABEL(GTK_BIN(menu_item)->child));

    win->color.ncolors = copy_colors_from_map (win->color.colors,
					       win->color.colors_size,
					       name,
					       color_map_list);
    draw_lsys (win);
}



void
on_lsys_win_change_colors (GtkMenuItem * menu_item, lsys_window *win)
{
    int i = 0;
    char * name;
    color_type type;

    if (!GTK_CHECK_MENU_ITEM(menu_item)->active)
	return;
    
    name = (char *) gtk_label_get_text (GTK_LABEL(GTK_BIN(menu_item)->child));

    if (g_ascii_strcasecmp (name, "Random Fade") == 0)
        type = COLOR_SMOOTH;
    else if (g_ascii_strcasecmp (name, "Spectrum") == 0)
	type = COLOR_UNIFORM;
    else
	type = COLOR_RANDOM;

    win->color.ncolors = get_colors (win->color.colors,
				     win->color.colors_size,
				     type,
				     i);
    draw_lsys (win);
}



void on_lsys_win_change_single_color (GtkMenuItem * menu_item,
				      lsys_window *win)
{

    GtkWidget *color_dlg = gtk_color_selection_dialog_new ("Select Foreground Color");
    gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG(color_dlg)->help_button);
    gtk_window_set_modal (GTK_WINDOW(color_dlg), TRUE);

    
    gtk_color_selection_set_current_color (
	GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_dlg)->colorsel),
	&(win->color.fgcolor));
    

    if (gtk_dialog_run (GTK_DIALOG(color_dlg)) == GTK_RESPONSE_OK)
    {
	gtk_color_selection_get_current_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(color_dlg)->colorsel),
					       &(win->color.fgcolor));

	win->color.ncolors = 1;
	win->color.colors[0] = win->color.fgcolor;

	draw_lsys (win);
    }
    gtk_widget_destroy (color_dlg);
}



void
on_lsys_win_change_modes (GtkMenuItem * menu_item, lsys_window *win)
{
    char * name;
   
    if (!GTK_CHECK_MENU_ITEM(menu_item)->active)
	return;

    name = (char *) gtk_label_get_text (GTK_LABEL(GTK_BIN(menu_item)->child));

    if (g_ascii_strcasecmp (name, "Random") == 0)
	win->mode = MODE_RANDOM;
    else if (g_ascii_strcasecmp (name, "Lines") == 0)
	win->mode = MODE_LINES;
    else
	win->mode = MODE_POINTS;

    draw_lsys (win);
}


void
on_lsys_win_change_background (GtkMenuItem *menu_item, lsys_window *win)
{
    GtkWidget *color_dlg = gtk_color_selection_dialog_new ("Select Background Color");
    gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG(color_dlg)->help_button);
    gtk_window_set_modal (GTK_WINDOW(color_dlg), TRUE);
    gtk_color_selection_set_current_color (
	GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_dlg)->colorsel),
	&(win->color.bgcolor));
    
    if (gtk_dialog_run (GTK_DIALOG(color_dlg)) == GTK_RESPONSE_OK)
    {
	gtk_color_selection_get_current_color (GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(color_dlg)->colorsel),
					       &(win->color.bgcolor));
	options.bgcolor = win->color.bgcolor;
	draw_lsys (win);
    }
    gtk_widget_destroy (color_dlg);
}



void
on_lsys_win_save_image (GtkMenuItem *menu_item, lsys_window *win)
{
    char *fname;
    GtkWidget * filesel;
    
    filesel = gtk_file_selection_new("Save L-system Image");
    gtk_window_set_modal (GTK_WINDOW (filesel), TRUE);

fname=	g_strdup_printf("%s%s%s.png",
			options.save_dir,
#ifdef WINDOWS
   "\\",
#else
   "",
#endif
			gtk_window_get_title(GTK_WINDOW(win->win)));
    gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), fname);
    g_free (fname);
    
    /* Replace OK button with "Save" button */
    gtk_widget_destroy (GTK_FILE_SELECTION(filesel)->ok_button);
    gtk_dialog_add_button(GTK_DIALOG(filesel),
			  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT);

    /* Loop until the user hits cancel or we get a valid filename to save */
    while (1)
    {
	const char *chosen;

	if (gtk_dialog_run (GTK_DIALOG(filesel)) != GTK_RESPONSE_ACCEPT)
	    break;

	chosen = gtk_file_selection_get_filename (GTK_FILE_SELECTION(filesel));

	/* Do some sanity checks */
	if (g_file_test (chosen, G_FILE_TEST_IS_DIR))
	{
	    char * s = g_strdup_printf ("Error: '%s' is a directory", chosen);
	    ok_dialog ("Error", s, GTK_WINDOW(filesel));
	    g_free (s);
	}
	else if (g_file_test (chosen, G_FILE_TEST_EXISTS))
	{
	    GtkWidget * dialog;
	    char * s = g_strdup_printf("File '%s' exists, overwrite?", chosen);
	    dialog = yes_no_dialog ("File exists", s, GTK_WINDOW(filesel));
	    g_free (s);
	   
	    if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
	    {	
		gtk_widget_destroy (dialog);
		save_image (chosen, win, filesel);
		break;
	    }
	    
	    gtk_widget_destroy (dialog);
	}
	else
	{
	    save_image (chosen, win, filesel);
	    break;
	}
    }

    gtk_widget_destroy (filesel);
}



/* Actually save the L-system image */
void
save_image (const char * file, lsys_window *win, GtkWidget *filesel)
{
    gboolean result;
    char * path_lower;
    GError * err = NULL;
    GdkPixbuf * pixbuf;
    enum {JPEG, PNG} save_type = PNG;
    
    path_lower = g_ascii_strdown (file, strlen(file));

    if (ENDS_WITH(path_lower, ".jpg") || ENDS_WITH(path_lower, ".jpeg"))
	save_type = JPEG;
    g_free (path_lower);
    
    pixbuf = gdk_pixbuf_get_from_drawable (
	NULL,
	win->pixmap,
	NULL,
	0, 0, 0, 0,
	win->drawing_area->allocation.width,
	win->drawing_area->allocation.height);
    
    /* There should be a custom "image save" dialog with GTK, so that the
     * users could actually choose these options easily... */
    if (save_type == PNG)
	result = gdk_pixbuf_save (pixbuf, file, "png", &err, NULL);
    else
	result = gdk_pixbuf_save (pixbuf, file, "jpeg", &err, 
				  "quality", "100", NULL);
    
    if (! result)
    {
	char * s = g_strdup_printf ("Error saving image: %s", file);
	ok_dialog (s, err->message, GTK_WINDOW(filesel));
	g_free (s);
	if (err) g_error_free (err);
	err = NULL;
    }

    g_free (options.save_dir);

/* don't ask, don't tell */
#ifdef WINDOWS
    options.save_dir = g_path_get_dirname(file);
#else
    {
      char * d = g_path_get_dirname(file);
      options.save_dir = g_strdup_printf("%s/", d);
      g_free (d);
    }
#endif
}

