/* lsyswin_ui.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Create the interface for an L-system window.
 */

#include "lsyswin_ui.h"
#include "lsyswin_cb.h"
#include "dialog_cb.h"
#include "colors.h"
#include "constants.h"
#include "util.h"

extern struct _opts options;
extern GList * color_map_list;

/* Create a L-system window, multiple numbers of these may be launched (and may
 * exist at the same time).  They get created when the user clicks on the
 * "Display" button on the main window
 */
lsys_window*
create_lsys_window(main_window * win)
{
    int width, height, i;
    GtkWidget *vbox, *table, *button_box, *button, *align;
    GtkObject * adj;
    lsys_window * lsys_win = (lsys_window *) g_malloc (sizeof(lsys_window));

    lsys_win->main_win = win;
    lsys_win->tooltips = gtk_tooltips_new();
    g_object_ref (lsys_win->tooltips);
    gtk_object_sink (GTK_OBJECT (lsys_win->tooltips));
    lsys_win->accel_group = gtk_accel_group_new ();

    width = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_width));
    height = (int) gtk_spin_button_get_value (GTK_SPIN_BUTTON(win->s_height));

    lsys_win->win = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    /* Center (or don't) the new L-system window */
    gtk_window_set_position (GTK_WINDOW (lsys_win->win), 
			     options.center_window ? GTK_WIN_POS_CENTER : GTK_WIN_POS_NONE);
    gtk_window_add_accel_group (GTK_WINDOW(lsys_win->win), lsys_win->accel_group);
    gtk_window_set_title (GTK_WINDOW(lsys_win->win),
			  gtk_label_get_text (GTK_LABEL(GTK_BIN(win->option_lsys)->child)));
    
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (lsys_win->win), vbox);

    create_lsys_window_menubar (lsys_win);
    gtk_box_pack_start (GTK_BOX (vbox),
			lsys_win->menu_bar,
			FALSE, FALSE, 0);

    table = gtk_table_new (2, 6, FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 8);
    gtk_table_set_row_spacings (GTK_TABLE (table), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table), 4);
    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, TRUE, 0);

    table_attach_label (GTK_TABLE(table), "Border:", 0, 1, 0, 1);
    table_attach_label (GTK_TABLE(table),  "Noise:", 2, 3, 0, 1);
    table_attach_label (GTK_TABLE(table),  "Level:", 4, 5, 0, 1);

    /* Border spin button */
    adj = gtk_adjustment_new (MIN(15, MIN(width,height)/6), 0, MIN(width,height)/3, 1, 10, 10);
    lsys_win->s_border = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
    gtk_tooltips_set_tip (lsys_win->tooltips, lsys_win->s_border,
			  "Size of border around L-system", NULL);
    gtk_table_attach (GTK_TABLE (table), lsys_win->s_border, 1, 2, 0, 1,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (lsys_win->s_border), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (lsys_win->s_border), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (lsys_win->s_border), TRUE);
    
    /* Noise spin button */
    adj = gtk_adjustment_new (0, 0, 1, 0.01, 10, 10);
    lsys_win->s_noise = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
    gtk_tooltips_set_tip (lsys_win->tooltips, lsys_win->s_noise,
			  "Amount of uniform random noise", NULL);
    gtk_table_attach (GTK_TABLE (table), lsys_win->s_noise, 3, 4, 0, 1,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (lsys_win->s_noise), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (lsys_win->s_noise), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (lsys_win->s_noise), TRUE);
    gtk_spin_button_set_digits (GTK_SPIN_BUTTON (lsys_win->s_noise), 2);
    
    /* Level spin button */
    adj = gtk_adjustment_new ((int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(win->s_level)), 0,
			      MAXLEVELSIZE-1, 1, 10, 10);
    lsys_win->s_level = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
    gtk_tooltips_set_tip (lsys_win->tooltips, lsys_win->s_level,
			  "Level at which to draw L-system (warning: don't set too high)", NULL);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (lsys_win->s_level), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (lsys_win->s_level), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (lsys_win->s_level), TRUE);
    gtk_table_attach (GTK_TABLE (table), lsys_win->s_level, 5, 6, 0, 1,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0); 
    
    /* Create Previous, Current, and Next Draw buttons */
    button_box = gtk_hbutton_box_new();
    gtk_container_set_border_width (GTK_CONTAINER (button_box), 4);
    gtk_table_attach (GTK_TABLE (table), button_box, 0, 6, 1, 2,
		      GTK_FILL,
		      (GTK_EXPAND | GTK_FILL), 0, 0);

    button = button_new_from_stock (GTK_STOCK_GO_DOWN, "_Previous", TRUE);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_tooltips_set_tip (lsys_win->tooltips, button,
			  "Draw L-system at previous level", NULL);
    g_signal_connect (G_OBJECT(button), "clicked",
			G_CALLBACK(on_lsys_win_draw_previous), lsys_win);
    gtk_container_add (GTK_CONTAINER(button_box), button);
 
    button = gtk_button_new_with_mnemonic ("_Draw");
    gtk_container_add (GTK_CONTAINER(button_box), button);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_tooltips_set_tip (lsys_win->tooltips, button,
			  "Draw L-system at current level", NULL);
    g_signal_connect (G_OBJECT(button), "clicked",
				G_CALLBACK(on_lsys_win_draw), lsys_win);

    button = button_new_from_stock (GTK_STOCK_GO_UP, "_Next", TRUE);
    gtk_container_add (GTK_CONTAINER(button_box), button);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_tooltips_set_tip (lsys_win->tooltips, button,
			  "Draw L-system at next level", NULL);
    g_signal_connect (G_OBJECT(button), "clicked",
			G_CALLBACK(on_lsys_win_draw_next), lsys_win);

    align = gtk_alignment_new (0.5, 0.5, 0, 0);
    lsys_win->drawing_area = gtk_drawing_area_new();
    gtk_widget_set_size_request (lsys_win->drawing_area, width, height);
    gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
    gtk_container_add (GTK_CONTAINER (align), lsys_win->drawing_area);

    g_signal_connect (G_OBJECT (lsys_win->win), "destroy",
			G_CALLBACK (on_lsys_win_destroy_event),
			lsys_win);

    /* make sure we get destroyed when parent (main) window closes */
    gtk_quit_add_destroy (1, GTK_OBJECT(lsys_win->win));

    g_signal_connect (G_OBJECT (lsys_win->win), "delete_event",
			G_CALLBACK (on_lsys_win_delete_event),
			NULL);

    g_signal_connect (G_OBJECT (lsys_win->drawing_area), "expose_event",
			(GtkSignalFunc) on_lsys_win_drawing_expose,
			lsys_win);

    lsys_win->width = width;
    lsys_win->height = height;

    lsys_win->color.colors_size = options.ncolors;
    lsys_win->color.colors = (GdkColor *)g_malloc(sizeof(GdkColor) * options.ncolors);

    lsys_win->mode = MODE_LINES;
    for (i = 0; i < MAXLEVELSIZE; ++i)
	lsys_win->levels[i].used = 0;
    lsys_win->pixmap = NULL;
    lsys_win->axiom = NULL;
    lsys_win->rule_string = NULL;

    lsys_win->color.ncolors = get_colors (lsys_win->color.colors,
					  options.ncolors,
					  COLOR_SMOOTH, 0);

    lsys_win->color.bgcolor = options.bgcolor;
    lsys_win->color.fgcolor.red = lsys_win->color.fgcolor.green = lsys_win->color.fgcolor.blue = 0;

    return lsys_win;
}



/* Create menubar for this L-system window */
void
create_lsys_window_menubar (lsys_window * win)
{
    int nitems;
    GtkItemFactory *item_factory;
    
    static GtkItemFactoryEntry menu_items[] =
    {
	{ "/_File", NULL, NULL, 0, "<Branch>" },
	{ "/_File/_Draw L-system", "<control>D" },
	{ "/File/_Save L-system Image", "<control>S", NULL, 0, "<StockItem>", GTK_STOCK_SAVE_AS },
	{ "/File/sep1", NULL, NULL, 0, "<Separator>" },
	{ "/File/_Close", "<control>W", NULL, 0, "<StockItem>", GTK_STOCK_CLOSE },
	{ "/_Mode", NULL, NULL, 0, "<Branch>" },
	{ "/Mode/Lines", "<control>L", NULL, 0, "<RadioItem>" },
	{ "/Mode/Points", "<control>P", NULL, 0, "/Mode/Lines" },
	{ "/Mode/Random", "<control>R", NULL, 0, "/Mode/Lines" },
	{ "/_Colors", NULL, NULL, 0, "<Branch>"},
	{ "/Colors/tearoff1", NULL, NULL, 0, "<Tearoff>"},
        { "/Colors/Single Foreground Color...", NULL, NULL, 0, "<StockItem>", GTK_STOCK_SELECT_COLOR},
	{ "/Colors/sep2", NULL, NULL, 0, "<Separator>" },
 	{ "/Colors/_Gradients", NULL, NULL, 0, "<Branch>"},
	{ "/Colors/Gradients/ctearoff1", NULL, NULL, 0, "<Tearoff>"},
	{ "/Colors/_Random Fade", "<control>F", NULL, 0, "<RadioItem>" },
	{ "/Colors/_Spectrum", "<control>U", NULL, 0, "/Colors/Random Fade" },
	{ "/Colors/Ra_ndom", "<control>N", NULL, 0, "/Colors/Random Fade" },

	{ "/Colors/sep1", NULL, NULL, 0, "<Separator>" },
	{ "/Colors/Change _Background Color...", "<control>B", NULL, 0, "<StockItem>", GTK_STOCK_SELECT_COLOR},
	{ "/_Help",               NULL,         0,    0,    "<Branch>" },
	{ "/Help/_Documentation...",      "F1"},
	{ "/Help/sep1", NULL, NULL, 0, "<Separator>" },
	{ "/Help/_About..."}
    };

    item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", win->accel_group);

    /* Set up item factory to go away with the window */
    g_object_ref (item_factory);
    gtk_object_sink (GTK_OBJECT (item_factory));
    g_object_set_data_full (G_OBJECT (win->win),
			    "<main>",
			    item_factory,
			    (GDestroyNotify) g_object_unref);    

    /* create menu items */
    gtk_item_factory_create_items (item_factory, G_N_ELEMENTS (menu_items),
				   menu_items, win->win);
    
    /* Add custom color menu entries to menu.  This is uglier than hell...
     * Blame factor:  GTK 95%, Me: 6%, Math teacher: 75% */
    if ((nitems = g_list_length (color_map_list)) < 1)
    {
	GtkItemFactoryEntry e = {"/Colors/Gradients/(Empty)",
				 NULL, NULL, 0, NULL, NULL};
	gtk_item_factory_create_item (item_factory, &e, NULL, 1);
    }
    else
    {
#define PER_GROUP 20
	int i, j = 0;
	
	nitems = g_list_length (color_map_list);
	
	for (i = 0; i < nitems; i+= PER_GROUP)
	{
	    char * menu_path;
	    GtkItemFactoryEntry e = {NULL, NULL, NULL, 0, NULL, NULL};

	    /* If we have more than 25 color items, split them into groups of
	     * PER_GROUP items */
	    if (nitems > 25)
	    {
		char first, second;

		/* Get the first letters of the first and last items within 
		 * this group of (at most, and usually) PER_GROUP items */
 		int last = ((i + PER_GROUP) > nitems) ? (nitems-1) : (i + PER_GROUP - 1);
		first = g_ascii_toupper(((color_group *)g_list_nth_data(color_map_list, i))->name[0]);
		second = g_ascii_toupper(((color_group *)g_list_nth_data(color_map_list, last))->name[0]);
		e.item_type = g_strdup_printf("<Branch>");
		menu_path = g_strdup_printf("/Colors/Gradients/_%c - %c",
					    first, second);
		e.path = menu_path;

		gtk_item_factory_create_item (item_factory, &e, NULL, 1);
		g_free (e.item_type);

		/* Have to do this to get rid of the underscore, stupid eh? */
 		g_free (menu_path);
		menu_path = g_strdup_printf("/Colors/Gradients/%c - %c",
					    first, second);

		e.item_type = "<Tearoff>";
		e.path = g_strdup_printf("%s/tearoff1", menu_path);
		gtk_item_factory_create_item (item_factory, &e, NULL, 1);
		g_free (e.path);
	    }
	    else
	    {
		menu_path = g_strdup ("/Colors/Gradients");
	    }

	    /* Add actual custom gradients */
	    e.item_type = "/Colors/Random Fade";
	    for (j = 0; j < PER_GROUP && (i + j) < nitems; ++j)
	    {
		color_group * g = (color_group *)g_list_nth_data(color_map_list, i + j);
		e.path = g_strdup_printf("%s/_%s", menu_path, g->name);
		gtk_item_factory_create_item (item_factory, &e, NULL, 1);

		/* Get rid of underscore again, this is fucking stupid, GTK
		 * should treat:
		 *  /_PATH
		 * as equivalent to:
		 *  /PATH
		 *
		 * and this wouldn't be necessary.
		 *
		 * I'm perfectly able to make my code look like shit by itself
		 * without the help of GTK...
		 */
 		g_free (e.path);
		e.path = g_strdup_printf("%s/%s", menu_path, g->name);
		attach_callback_tooltip (
		    gtk_item_factory_get_item(item_factory, e.path),
		    "activate", G_CALLBACK(on_lsys_win_change_colors_map),
		    win, CONNECT_NORMAL,
		    win->tooltips, g->name);
		g_free (e.path);
	    }
	    g_free (menu_path);
	}
    }
    
    /* Attach signals and tooltips to the menu items */
    attach_callback_tooltip (
	GTK_WIDGET(gtk_item_factory_get_item(item_factory, "/File/Draw L-system")),
	"activate",
	G_CALLBACK(on_lsys_win_draw), win, CONNECT_NORMAL,
	win->tooltips, "Draw L-system at current level");
    
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/File/Save L-system Image"),
	"activate", G_CALLBACK(on_lsys_win_save_image), win, CONNECT_NORMAL,
	win->tooltips, "Save L-system image");

     attach_callback_tooltip (
 	 GTK_WIDGET(gtk_item_factory_get_item(item_factory, "/File/Close")),
	 "activate",
	 G_CALLBACK(gtk_widget_destroy), win->win, CONNECT_SWAPPED,
	 win->tooltips, "Close window");
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Mode/Lines"),
	 "activate", G_CALLBACK(on_lsys_win_change_modes), win, CONNECT_NORMAL,
	 win->tooltips, "Draw L-system using lines (default)");
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Mode/Points"),
	 "activate", G_CALLBACK(on_lsys_win_change_modes), win, CONNECT_NORMAL,
	 win->tooltips, "Draw L-system using points");
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Mode/Random"),
	 "activate", G_CALLBACK(on_lsys_win_change_modes), win, CONNECT_NORMAL,
	 win->tooltips, "Draw L-system using lines and points, chosen randomly");

     
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Colors/Single Foreground Color..."),
	 "activate", G_CALLBACK(on_lsys_win_change_single_color), win, CONNECT_NORMAL,
	 win->tooltips, "Choose a single foreground color");
     

     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Colors/Gradients"),
	 NULL, NULL, win, CONNECT_NORMAL,
	 win->tooltips, "Choose specific gradient coloring");
     
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Colors/Random Fade"),
	 "activate", G_CALLBACK(on_lsys_win_change_colors), win, CONNECT_NORMAL,
	 win->tooltips, "Choose a random fade coloring");
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Colors/Spectrum"),
	 "activate", G_CALLBACK(on_lsys_win_change_colors), win, CONNECT_NORMAL,
	 win->tooltips, "Choose a color spectrum coloring");
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Colors/Random"),
	 "activate", G_CALLBACK(on_lsys_win_change_colors), win, CONNECT_NORMAL,
	 win->tooltips, "Choose a random coloring");
     
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Colors/Change Background Color..."),
	 "activate", G_CALLBACK(on_lsys_win_change_background), win, CONNECT_NORMAL,
	 win->tooltips, "Change the background color");

     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Help/Documentation..."),
	 "activate", G_CALLBACK(on_documentation), win->win, CONNECT_NORMAL,
	 win->tooltips, "View documentation");
     attach_callback_tooltip (
	 gtk_item_factory_get_item(item_factory, "/Help/About..."),
	 "activate", G_CALLBACK(on_about), win->win, CONNECT_NORMAL,
	 win->tooltips, "About L-system Explorer");
   
     win->menu_bar = gtk_item_factory_get_widget (item_factory, "<main>");
}

