/* mainwin_cb.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Callback functions for the main window and its sub-windows.
 *
 */

#include "mainwin_cb.h"
#include "lsyswin_ui.h"
#include "dialog_ui.h"
#include "lsys_draw.h"
#include "lsys.h"
#include "util.h"
#include "config.h"

extern struct _opts options;
extern path_info paths;

/* -----------------------------------------------------------------------
 * 
 *  Main window callbacks
 *
 * -----------------------------------------------------------------------
 */

/* Exit program */
void
on_main_win_destroy_event (GtkWidget *w, main_window * win)
{
    int i;
    for (i = 0; i < g_list_length(win->icons); ++i)
    {
        GdkPixbuf * pb = (GdkPixbuf *)g_list_nth_data(win->icons, i);
        g_object_unref (pb);
    }
    if (options.save_dir) g_free (options.save_dir);
    if (win->tooltips) g_object_unref(win->tooltips);
    g_free (win);
    gtk_main_quit();
}



/* Destroy ourselves */
gboolean
on_main_win_delete_event(GtkWidget *w, GdkEvent *event, 
				  gpointer user_data)
{
    gtk_widget_destroy (w);
    return FALSE;
}



/* Note: the first argument (w) has no meaning here, it could be anything, so
 * don't use it!  Everything necessary is in "win" as it is */
void
on_main_win_display_clicked (GtkWidget *w, main_window * win)
{
    lsys_window * lsys_win;
    lsys_win = create_lsys_window(win);

    if (get_current_lsys (lsys_win))
    {
	gtk_widget_destroy(lsys_win->win);
	return;
    }

    gtk_widget_show_all (lsys_win->win);
    draw_lsys(lsys_win);
}



/* Display all L-systems in currently selected group */
void on_main_win_display_all_in_group_clicked (GtkWidget *w, main_window * win)
{
    int i, n, current;
    char * group_name = (char *) gtk_label_get_text(GTK_LABEL(GTK_BIN(win->option_group)->child));

    if ((n = get_lsys_count (group_name)) < 0) return;

    current = gtk_option_menu_get_history (GTK_OPTION_MENU(win->option_lsys));

    /* GTK is SO fucked WRT the option menu API... */
    for (i = n; i >= 0; --i)
    {
	gtk_option_menu_set_history (GTK_OPTION_MENU(win->option_lsys), i);
	on_main_win_display_clicked (w, win);
    }

    /* Restore selected fractal */
    gtk_option_menu_set_history (GTK_OPTION_MENU(win->option_lsys), current);
}



void
on_main_win_option_group_changed (GtkWidget *w, main_window * win)
{
    char * group_name;

    group_name = (char *) gtk_label_get_text(GTK_LABEL(GTK_BIN(win->option_group)->child));

    /* create new option menu with lsys corresponding to this group */
    gtk_option_menu_set_menu(GTK_OPTION_MENU(win->option_lsys),
			     get_lsys_names_menu(group_name));


    on_main_win_new_lsys_selected (
	group_name,
	(char*) gtk_label_get_text (GTK_LABEL(GTK_BIN(win->option_lsys)->child)),
	win);
}



void
on_main_win_option_lsys_changed (GtkWidget *w, main_window * win)
{
    on_main_win_new_lsys_selected (
	(char *) gtk_label_get_text (GTK_LABEL(GTK_BIN(win->option_group)->child)),
	(char *) gtk_label_get_text (GTK_LABEL(GTK_BIN(win->option_lsys)->child)),
	win);
}



void
on_main_win_options (GtkWidget *w, main_window *win)
{
    int rc;
    options_window * opt_win = create_options_dialog (win);
    gtk_widget_show_all (opt_win->dlg);

    opt_win->pixmap = gdk_pixmap_new (opt_win->bgcolor->window,
				      opt_win->bgcolor->allocation.width,
				      opt_win->bgcolor->allocation.height,
				      -1);
    opt_win->gc = gdk_gc_new (opt_win->bgcolor->window);

    gdk_gc_set_rgb_fg_color (opt_win->gc, &(opt_win->defaults.bgcolor));
    gdk_draw_rectangle (opt_win->pixmap, opt_win->gc,
			TRUE,
			0, 0,
			opt_win->bgcolor->allocation.width,
			opt_win->bgcolor->allocation.height);
    gdk_draw_drawable (opt_win->bgcolor->window,
		       opt_win->gc,
		       opt_win->pixmap,
		       0, 0, 0, 0,
		       opt_win->bgcolor->allocation.width,
		       opt_win->bgcolor->allocation.height);

    while (1)
    {
	if (gtk_dialog_run (GTK_DIALOG(opt_win->dlg)) != GTK_RESPONSE_ACCEPT)
	    break;

	opt_win->defaults.width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(opt_win->s_width));
	opt_win->defaults.height = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(opt_win->s_height));
        opt_win->defaults.ncolors = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(opt_win->s_colors));
	opt_win->defaults.center_window = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(opt_win->chk_center_windows));

	gtk_spin_button_set_value (GTK_SPIN_BUTTON(win->s_height),
				   opt_win->defaults.height);
	gtk_spin_button_set_value (GTK_SPIN_BUTTON(win->s_width),
				   opt_win->defaults.width);
	
	if ((rc = save_user_options (&opt_win->defaults)))
	{
	    char * s = g_strdup_printf ("Error saving options to '%s': %s",
					paths.options_file, g_strerror(rc));
	    ok_dialog ("Error", s, GTK_WINDOW(opt_win->dlg));
	    g_free (s);
	}
	else
	{
	    g_free (options.save_dir);
	    options = opt_win->defaults;
	    break;
	}
    }

    gtk_widget_destroy (opt_win->dlg);
}


void
on_main_win_save_lsystem (GtkWidget *w, main_window * win)
{
    save_lsys_window * save_win = create_save_lsys_window(win);
    gtk_window_set_modal (GTK_WINDOW(save_win->win), TRUE);
    gtk_widget_show_all (save_win->win);
}



void
on_main_win_new_lsys_selected (char * group_name, char * lsys_name,
			       main_window * win)
{
    char *axiom, *rules;
    double da, a0, ds;
    int level;

    get_lsys_info (group_name, lsys_name,
		      &axiom, &rules, &da, &a0, &level, &ds);

    gtk_entry_set_text(GTK_ENTRY(win->axiom), axiom);
    set_textview_text (win->rules, rules);
    g_free (rules);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win->s_a0), a0);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win->s_da), da);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win->s_level), level);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win->s_ds), ds);
}




/* Save the current L-system (not the image, the L-system itself) */
void
on_save_lsys_win_response (GtkWidget * dialog, gint result,
			   save_lsys_window * save_win)
{
    int rc;
    char *lsys_name, *group_name, *s1, *s2;

    if (result != GTK_RESPONSE_ACCEPT)
    {
	gtk_widget_destroy (dialog);
	return;
    }

    /* Get name of lsys and lsys group the user wants to save */
    lsys_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(save_win->entry_lsys)));
    group_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(save_win->combo_group)->entry)));

    g_strstrip (lsys_name);
    g_strstrip (group_name);
    g_strdelimit (lsys_name, ":", ' ');
    g_strdelimit (group_name, ":", ' ');

    if (! group_name[0])
    {
	ok_dialog ("Empty group name",
		   "Error: You must choose a group name to save under",
		   GTK_WINDOW(save_win->main_win->win));
	g_free (lsys_name);
	g_free (group_name);
	return;
    }
    if (! lsys_name[0])
    {
	ok_dialog ("Empty L-system name",
		   "Error: You must choose a name for the L-system to save",
		   GTK_WINDOW(save_win->main_win->win));
	g_free (lsys_name);
	g_free (group_name);
	return;
    }
    if (lsys_exists (group_name, lsys_name))
    {
	int response;
	GtkWidget * dialog;
	char * s;
	s = g_strdup_printf("L-system '%s' in group '%s' already exists, overwrite?", lsys_name, group_name);
	dialog = yes_no_dialog ("Overwrite L-system?", s, GTK_WINDOW(save_win->main_win->win));
	response = gtk_dialog_run (GTK_DIALOG(dialog));
	gtk_widget_destroy (dialog);
	g_free (s);
	if (response == GTK_RESPONSE_REJECT)
	    return;
    }

    if ((rc = save_lsys (
	    group_name,
	    lsys_name,
	    s1 = g_strdup(gtk_entry_get_text(GTK_ENTRY(save_win->main_win->axiom))),
	    s2 = get_textview_text(save_win->main_win->rules),
	    gtk_spin_button_get_value (GTK_SPIN_BUTTON(save_win->main_win->s_a0)),
	    gtk_spin_button_get_value (GTK_SPIN_BUTTON(save_win->main_win->s_da)),
	    gtk_spin_button_get_value (GTK_SPIN_BUTTON(save_win->main_win->s_ds)),
	    (int)gtk_spin_button_get_value (GTK_SPIN_BUTTON(save_win->main_win->s_level))
	     )))
    {
	char * s = g_strdup_printf ("Error saving L-system to '%s': %s",
			     paths.custom_file, g_strerror(rc));

	ok_dialog ("Error", s, GTK_WINDOW(save_win->win));
	g_free (s);
	return;
    }
    
    gtk_option_menu_set_menu(GTK_OPTION_MENU(save_win->main_win->option_group),
			     get_group_names_menu ());
    on_main_win_option_group_changed (save_win->main_win->option_group,
				      save_win->main_win);
    g_free (lsys_name);
    g_free (group_name);
    g_free (s1);
    g_free (s2);

    gtk_widget_destroy (dialog);
}


void on_options_dlg_browse (GtkWidget * dialog, options_window * win)
{
    char *path;
    GtkWidget *filesel = gtk_file_selection_new("Image save directory");
    gtk_window_set_modal (GTK_WINDOW (filesel), TRUE);

    path = (char *)gtk_entry_get_text(GTK_ENTRY(win->entry_path));

    if (!g_file_test(path, G_FILE_TEST_IS_DIR))
	path = win->defaults.save_dir;
    gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), path);

    while (1)
    {
	char *chosen;
	if (gtk_dialog_run (GTK_DIALOG(filesel)) != GTK_RESPONSE_OK)
            break;
	chosen = g_strdup(gtk_file_selection_get_filename (GTK_FILE_SELECTION(filesel)));

/* This is asinine, glib doesn't think C:\foo\bar\ is a directory, but it
 * thinks C:\foo\bar and C:\foo\bar\. are... */
#ifdef WINDOWS
   if (chosen[strlen(chosen)-1] == '\\') chosen[strlen(chosen)-1] = 0;
#endif

	if (! g_file_test(chosen, G_FILE_TEST_IS_DIR))
	{
	    char * s = g_strdup_printf ("Error: '%s' is not a directory", chosen);
	    ok_dialog ("Error", s, GTK_WINDOW(filesel));
	    g_free (s);
	    g_free (chosen);
	}
	else
	{
	    gtk_entry_set_text(GTK_ENTRY(win->entry_path), chosen);
	    g_free (win->defaults.save_dir);
	    win->defaults.save_dir = chosen;
	    break;
	}
    }

    gtk_widget_destroy (filesel);
}



gboolean on_options_dlg_change_bgcolor (GtkWidget * dialog,
					GdkEventButton *event,
					options_window *win)
{
    GtkWidget * color_dlg;
    
    color_dlg = gtk_color_selection_dialog_new ("Select Background Color");
    gtk_window_set_modal (GTK_WINDOW(color_dlg), TRUE);
    gtk_widget_destroy (GTK_COLOR_SELECTION_DIALOG(color_dlg)->help_button);

    gtk_color_selection_set_current_color (
	GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (color_dlg)->colorsel),
	&(win->defaults.bgcolor));
    
    if (gtk_dialog_run (GTK_DIALOG(color_dlg)) == GTK_RESPONSE_OK)
    {
	gtk_color_selection_get_current_color (
	    GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(color_dlg)->colorsel),
	    &win->defaults.bgcolor);
	
	gdk_gc_set_rgb_fg_color (win->gc, &win->defaults.bgcolor);
	gdk_draw_rectangle (win->pixmap, win->gc,
			    TRUE,
			    0, 0,
			    win->bgcolor->allocation.width,
			    win->bgcolor->allocation.height);
	
	gdk_draw_drawable (win->bgcolor->window,
			   win->gc,
			   win->pixmap,
			   0, 0, 0, 0,
			   win->bgcolor->allocation.width,
			   win->bgcolor->allocation.height);
    }
    gtk_widget_destroy (color_dlg);
    return FALSE;
}

gint
on_option_dlg_bgcolor_expose (GtkWidget * widget, GdkEventExpose * event, 
			      options_window * win)
{
    gdk_draw_drawable (win->bgcolor->window,
		       win->gc,
		       win->pixmap,
		       event->area.x, event->area.y,
		       event->area.x, event->area.y,
		       event->area.width, event->area.height);
    return FALSE;
}


