/* mainwin_ui.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Create interface for main window and it's sub-dialogs.
 */

#include "mainwin_ui.h"
#include "mainwin_cb.h"
#include "dialog_cb.h"
#include "icons.h"
#include "lsys.h"
#include "util.h"
#include "constants.h"

extern path_info paths;
extern struct _opts options;
extern GList * groups;

/* Create the main window, which the user sees at startup, and attach all
 * the signal handlers, tooltips, etc. */
main_window*
create_main_window()
{
    GtkWidget *vbox, *table, *frame, *sep, *button_box, *scroller;
    GtkObject *adj;

    main_window * win = (main_window *) g_malloc (sizeof(main_window));

    win->win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW(win->win), TITLE);

    win->tooltips = gtk_tooltips_new();
    g_object_ref (win->tooltips);
    gtk_object_sink (GTK_OBJECT (win->tooltips));

    win->accel_group = gtk_accel_group_new ();
    gtk_window_add_accel_group (GTK_WINDOW(win->win), win->accel_group);

    /* for menubar, frame, and "Display" button */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER(win->win), vbox);
    
    /* Add menubar */
    create_main_window_menubar (win);
    gtk_box_pack_start (GTK_BOX (vbox),
			win->menu_bar,
			FALSE, FALSE, 0);

    /* Create and add frame */
    frame = gtk_frame_new("L-system Options");
    gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
    gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
 
    /* Create table */
    table = gtk_table_new (10, 4, FALSE);
    gtk_container_add (GTK_CONTAINER (frame), table);
    gtk_container_set_border_width (GTK_CONTAINER (table), 4);
    gtk_table_set_row_spacings (GTK_TABLE (table), 4);
    
    /* Add all labels to table */
    table_attach_label (GTK_TABLE(table),         "Group:", 0, 1, 0, 1);
    table_attach_label (GTK_TABLE(table),      "L-system:", 0, 1, 1, 2);
    table_attach_label (GTK_TABLE(table),         "Axiom:", 0, 1, 2, 3);
    table_attach_label (GTK_TABLE(table),         "Rules:", 0, 1, 3, 4);
    table_attach_label (GTK_TABLE(table), "Initial Angle:", 0, 1, 6, 7);
    table_attach_label (GTK_TABLE(table),    "Turn Angle:", 0, 1, 7, 8);
    table_attach_label (GTK_TABLE(table),         "Level:", 2, 3, 6, 7);
    table_attach_label (GTK_TABLE(table),         "Scale:", 2, 3, 7, 8);
    table_attach_label (GTK_TABLE(table),         "Width:", 0, 1, 9, 10);
    table_attach_label (GTK_TABLE(table),        "Height:", 2, 3, 9, 10);
  
    /* Add all other widgets to table */

    /* option drop down menus and text boxes */
    win->option_group = gtk_option_menu_new();
    gtk_tooltips_set_tip (win->tooltips, win->option_group,
			  "L-system group to select from", NULL);
    win->option_lsys = gtk_option_menu_new();
    gtk_tooltips_set_tip (win->tooltips, win->option_lsys,
			  "L-system to use", NULL);

    gtk_table_attach (GTK_TABLE (table),
		      win->option_group,
		      1, 3, 0, 1,
		      GTK_FILL,
		      0, 0, 0);
    gtk_table_attach (GTK_TABLE (table),
		      win->option_lsys,
		      1, 3, 1, 2,
		      GTK_FILL,
		      0, 0, 0);
    
    /* Create and attach text boxes */
    win->axiom = gtk_entry_new ();
    gtk_tooltips_set_tip (win->tooltips, win->axiom,
			  "L-system axiom", NULL);
    gtk_table_attach (GTK_TABLE (table),
		      win->axiom,
		      1, 4, 2, 3,
		      (GTK_EXPAND | GTK_FILL),
		      0, 0, 0);
    scroller = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroller),
		    			GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scroller),
					 GTK_SHADOW_IN);
    win->rules = gtk_text_view_new();

    /* This doesn't work, and it doesn't work when I try sticking the tooltip
     * on the scroll window too.  Nifty eh? */
    gtk_tooltips_set_tip (win->tooltips, win->rules,
			  "L-system rules (one rule per line)", NULL);

    gtk_container_add (GTK_CONTAINER (scroller), win->rules);
    gtk_table_attach (GTK_TABLE (table),
		      scroller,
		      1, 4, 3, 5,
		      (GTK_EXPAND | GTK_FILL),
		      (GTK_EXPAND | GTK_FILL), 0, 0);

    sep = gtk_hseparator_new();
    gtk_table_attach (GTK_TABLE (table), sep,
		      0, 4, 5, 6,
		      GTK_FILL,
		      (GTK_EXPAND | GTK_FILL), 0, 2);

    /* Load user options (width, height, background color, save path).
     * "defaults" is whatever is in ~/.lsysexp/options, "options" contains
     * the state for the current session, which may be different */
    load_user_options (&options);
    
    /* Spin buttons */
    adj = gtk_adjustment_new (90, 0, 360, 0.05, 10, 10);
    win->s_a0 = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 1);
    gtk_tooltips_set_tip (win->tooltips, win->s_a0,
			  "Initial angle, in degrees (0 is north, 90 east, etc.)",
			  NULL);
    gtk_table_attach (GTK_TABLE (table), win->s_a0, 1, 2, 6, 7,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    
    adj = gtk_adjustment_new (-360, -360, 360, 0.05, 10, 10);
    win->s_da = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 3);
    gtk_tooltips_set_tip (win->tooltips, win->s_da,
			  "Turn angle, in degrees", NULL);
    gtk_table_attach (GTK_TABLE (table), win->s_da, 1, 2, 7, 8,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);

    adj = gtk_adjustment_new (1, 0, MAXLEVELSIZE-1, 1, 10, 10);
    win->s_level = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_tooltips_set_tip (win->tooltips, win->s_level,
			  "L-system recursion level (warning: don't set too high!)", NULL);
    gtk_table_attach (GTK_TABLE (table), win->s_level, 3, 4, 6, 7,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    
    adj = gtk_adjustment_new (0.5, 0, 1, 0.01, 10, 10);
    win->s_ds = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 2);
    gtk_tooltips_set_tip (win->tooltips, win->s_ds,
			  "Scale factor (relevant only if '%' is present)", NULL);
    gtk_table_attach (GTK_TABLE (table), win->s_ds, 3, 4, 7, 8,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);

    sep = gtk_hseparator_new();
    gtk_table_attach (GTK_TABLE (table), sep,
		      0, 4, 8, 9,
		      GTK_FILL,
		      (GTK_EXPAND | GTK_FILL), 0, 2);

    adj = gtk_adjustment_new (MIN(gdk_screen_width(), options.width), 16, gdk_screen_width(), 1, 10, 10);
    win->s_width = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_tooltips_set_tip (win->tooltips, win->s_width,
			  "Width of L-system in pixels", NULL);
    gtk_table_attach (GTK_TABLE (table), win->s_width, 1, 2, 9, 10,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);

    adj = gtk_adjustment_new (MIN(gdk_screen_height(), options.height), 16, gdk_screen_height(), 1, 10, 10);
    win->s_height = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_tooltips_set_tip (win->tooltips, win->s_height,
			  "Height of L-system in pixels", NULL);
    gtk_table_attach (GTK_TABLE (table), win->s_height, 3, 4, 9, 10,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);

    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (win->s_a0), TRUE);
    gtk_spin_button_set_digits (GTK_SPIN_BUTTON (win->s_a0), 2);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (win->s_a0), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (win->s_da), TRUE);
    gtk_spin_button_set_digits (GTK_SPIN_BUTTON (win->s_da), 2);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (win->s_da), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (win->s_level), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (win->s_level), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (win->s_ds), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (win->s_ds), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (win->s_width), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (win->s_width), GTK_UPDATE_IF_VALID);
    gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (win->s_height), TRUE);
    gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (win->s_height), GTK_UPDATE_IF_VALID);

    button_box = gtk_hbutton_box_new();
    gtk_box_pack_start (GTK_BOX (vbox), button_box, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (button_box), 3);

    win->b_display = gtk_button_new_with_label ("Display");
    gtk_tooltips_set_tip (win->tooltips, win->b_display,
			  "Display L-system", NULL);
    gtk_container_add (GTK_CONTAINER (button_box), win->b_display);
    GTK_WIDGET_SET_FLAGS (win->b_display, GTK_CAN_DEFAULT);

    /* Add icon list */
    win->icons = NULL;
    win->icons = g_list_append (win->icons, gdk_pixbuf_new_from_xpm_data(icon_16));
    win->icons = g_list_append (win->icons, gdk_pixbuf_new_from_xpm_data(icon_32));
    win->icons = g_list_append (win->icons, gdk_pixbuf_new_from_xpm_data(icon_48));
    win->icons = g_list_append (win->icons, gdk_pixbuf_new_from_xpm_data(icon_64));
    win->icons = g_list_append (win->icons, gdk_pixbuf_new_from_xpm_data(icon_128));
    gtk_window_set_default_icon_list(win->icons);
    
    /* Fill option menus with groups and L-systems */
    load_lsystems();
    gtk_option_menu_set_menu(GTK_OPTION_MENU(win->option_group),
			     get_group_names_menu ());
    gtk_option_menu_set_menu(GTK_OPTION_MENU(win->option_lsys),
			     get_lsys_names_menu(((lsys_group *)groups->data)->group_name));
    on_main_win_option_group_changed (win->option_group, win);
    
    /* Connect various callback functions */
    g_signal_connect (G_OBJECT (win->win), "destroy",
		      G_CALLBACK (on_main_win_destroy_event),
		      win);
    g_signal_connect (G_OBJECT (win->win), "delete_event",
		      G_CALLBACK (on_main_win_delete_event),
		      NULL);
    g_signal_connect (G_OBJECT (win->b_display), "clicked",
		      G_CALLBACK (on_main_win_display_clicked),
		      win);
    
    g_signal_connect (G_OBJECT (GTK_OPTION_MENU (win->option_group)),
		      "changed",
		      G_CALLBACK (on_main_win_option_group_changed),
		      win);
    g_signal_connect (G_OBJECT (GTK_OPTION_MENU (win->option_lsys)),
		      "changed",
		      G_CALLBACK (on_main_win_option_lsys_changed),
		      win);
    
    gtk_widget_grab_focus (win->b_display);
    return win;
}



void
create_main_window_menubar (main_window * win)
{
    GtkItemFactory *item_factory;
    static GtkItemFactoryEntry menu_items[] =
    {
	{ "/_File",               NULL,         0,    0,    "<Branch>" },
	{ "/File/_Save L-system", "<control>S", NULL, 0, "<StockItem>", GTK_STOCK_SAVE_AS },
	{ "/File/_Display L-system", "<control>D" },
	{ "/File/Display _All L-systems in Group", "<control>A" },
	{ "/File/sep1",           NULL,         NULL, 0,    "<Separator>" },
	{ "/File/Preferences...",   "<control>P" },
	{ "/File/sep2",           NULL,         NULL, 0,    "<Separator>" },
	{ "/File/_Quit",          "<control>Q", NULL, 0,    "<StockItem>", GTK_STOCK_QUIT },

	{ "/_Help",               NULL,         0,    0,    "<Branch>" },
	{ "/Help/_Documentation...",      "F1"},
	{ "/Help/sep1", NULL, NULL, 0, "<Separator>" },
	{ "/Help/_About..."}
    };

    item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", win->accel_group);

    /* Set up item factory to go away with the window */
    g_object_ref (item_factory);
    gtk_object_sink (GTK_OBJECT (item_factory));
    g_object_set_data_full (G_OBJECT (win->win),
			    "<main>",
			    item_factory,
			    (GDestroyNotify) g_object_unref);    

    /* create menu items */
    gtk_item_factory_create_items (item_factory, G_N_ELEMENTS (menu_items),
				   menu_items, win->win);

    /* establish callbacks and tooltips */
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/File/Save L-system"),
	"activate", G_CALLBACK(on_main_win_save_lsystem), win, CONNECT_NORMAL,
	win->tooltips, "Save L-system type");
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/File/Display L-system"),
	"activate", G_CALLBACK(on_main_win_display_clicked), win, CONNECT_NORMAL,
	win->tooltips, "Display current L-system");
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/File/Display All L-systems in Group"),
	"activate", G_CALLBACK(on_main_win_display_all_in_group_clicked), win, CONNECT_NORMAL,
	win->tooltips, "Display all L-systems in current group (which may be a lot, so be careful)");

    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/File/Quit"),
	"activate", G_CALLBACK(gtk_widget_destroy), win->win, CONNECT_SWAPPED,
	win->tooltips, "Quit application");
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/File/Preferences..."),
	"activate", G_CALLBACK(on_main_win_options), win, CONNECT_NORMAL,
	win->tooltips, "Edit user preferences");
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/Help/Documentation..."),
	"activate", G_CALLBACK(on_documentation), win->win, CONNECT_NORMAL,
	win->tooltips, "View documentation");
    attach_callback_tooltip (
	gtk_item_factory_get_item(item_factory, "/Help/About..."),
	"activate", G_CALLBACK(on_about), win->win, CONNECT_NORMAL,
	win->tooltips, "About L-system Explorer");
    
    win->menu_bar = gtk_item_factory_get_widget (item_factory, "<main>");
}



/* Create a simple dialog for the user to save the current L-system on the main
 * window */
save_lsys_window*
create_save_lsys_window(main_window * win)
{
    GtkWidget *table;
    save_lsys_window * save_win;

    save_win = (save_lsys_window *) g_malloc (sizeof(save_lsys_window));
    save_win->main_win = win;
    save_win->combo_group = gtk_combo_new();
    gtk_combo_set_popdown_strings (GTK_COMBO(save_win->combo_group),
				   get_group_names_list());

    save_win->entry_lsys = gtk_entry_new();
    save_win->win = gtk_dialog_new_with_buttons (
	"Save L-system",
	GTK_WINDOW(win->win),
	GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
	GTK_STOCK_SAVE,
	GTK_RESPONSE_ACCEPT,
	GTK_STOCK_CANCEL,
	GTK_RESPONSE_REJECT,
	NULL);

    gtk_dialog_set_default_response (GTK_DIALOG(save_win->win),
				     GTK_RESPONSE_ACCEPT);

    table = gtk_table_new (2, 2, FALSE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 5);

    /* add labels to table */
    gtk_table_attach (GTK_TABLE (table),
		      label_align_pad ("Group:", 1.0, 0.5, 8, 0),
		      0, 1, 0, 1,
		      GTK_FILL,
		      0, 0, 0);
    gtk_table_attach (GTK_TABLE (table),
		      label_align_pad ("L-system Name:", 1.0, 0.5, 8, 0),
		      0, 1, 1, 2,
		      GTK_FILL,
		      0, 0, 0);
    
    /* add combo and entry widgets to table */
    gtk_table_attach (GTK_TABLE (table), save_win->combo_group, 1, 2, 0, 1,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    gtk_table_attach (GTK_TABLE (table), save_win->entry_lsys, 1, 2, 1, 2,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(save_win->win)->vbox),
		       table);
    
    g_signal_connect (G_OBJECT (save_win->win),
			"response",
			G_CALLBACK(on_save_lsys_win_response),
			save_win);
    return save_win;
}



/* Create options dialog */
options_window *create_options_dialog (main_window *win)
{
    options_window * opt_win;
    GtkObject *adj;
    GtkWidget *table, *vbox, *hbox, *event_box;
    
    opt_win = (options_window *)g_malloc(sizeof(options_window));
    opt_win->tooltips = gtk_tooltips_new();
    load_user_options (&opt_win->defaults);

    opt_win->dlg = gtk_dialog_new_with_buttons (
	"Preferences",
	GTK_WINDOW(win->win),
	GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
	GTK_STOCK_SAVE,
	GTK_RESPONSE_ACCEPT,
	GTK_STOCK_CANCEL,
	GTK_RESPONSE_REJECT,
	NULL);

    gtk_window_set_resizable (GTK_WINDOW (opt_win->dlg), FALSE);

    vbox = GTK_DIALOG(opt_win->dlg)->vbox;
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    
    table = gtk_table_new (3, 2, FALSE);
    gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table), 8);
    gtk_table_set_row_spacings (GTK_TABLE (table), 4);
    gtk_table_set_col_spacings (GTK_TABLE (table), 4);
    table_attach_label (GTK_TABLE(table), "Width:", 0, 1, 0, 1);
    table_attach_label (GTK_TABLE(table), "Height:", 0, 1, 1, 2);
    table_attach_label (GTK_TABLE(table), "Colors:", 0, 1, 2, 3);

    /* Width spin button */
    adj = gtk_adjustment_new (MIN(gdk_screen_width(), opt_win->defaults.width), 16, gdk_screen_width(), 1, 10, 10);
    opt_win->s_width = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_tooltips_set_tip (opt_win->tooltips, opt_win->s_width,
			  "Width of L-system in pixels", NULL);
    gtk_table_attach (GTK_TABLE (table), opt_win->s_width, 1, 2, 0, 1,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);

    /* Height spin button */
    adj = gtk_adjustment_new (MIN(gdk_screen_height(), opt_win->defaults.height), 16, gdk_screen_height(), 1, 10, 10);
    opt_win->s_height = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_tooltips_set_tip (opt_win->tooltips, opt_win->s_height,
			  "Height of L-system in pixels", NULL);
    gtk_table_attach (GTK_TABLE (table), opt_win->s_height, 1, 2, 1, 2,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
		      (GtkAttachOptions) (0), 0, 0);

    /* Colors spin button */
    adj = gtk_adjustment_new (opt_win->defaults.ncolors, MINCOLORS, MAXCOLORS, 1, 10, 10);
    opt_win->s_colors = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
    gtk_tooltips_set_tip (opt_win->tooltips, opt_win->s_colors,
                          "Number of colors", NULL);
    gtk_table_attach (GTK_TABLE (table), opt_win->s_colors, 1, 2, 2, 3,
                      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);
    
    /* Center L-system Windows checkbox */
    opt_win->chk_center_windows = gtk_check_button_new_with_label ("Center L-system Windows");
    gtk_tooltips_set_tip (opt_win->tooltips, opt_win->chk_center_windows,
			  "Center New L-system Windows", NULL);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(opt_win->chk_center_windows),
			       opt_win->defaults.center_window);

    gtk_table_attach (GTK_TABLE (table),
		      opt_win->chk_center_windows,
		      0, 2, 3, 4,
		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);

    gtk_box_pack_start (GTK_BOX(hbox), gtk_vseparator_new(), TRUE, TRUE, 0);    
    /* Background color label & drawing area */
    gtk_box_pack_start (GTK_BOX(hbox),
			label_align_pad ("Background:",  1.0, 0.5, 8, 0),
			FALSE, FALSE, 0);

    opt_win->bgcolor = gtk_drawing_area_new();
    gtk_widget_set_size_request (opt_win->bgcolor, 32, 32);
    event_box = gtk_event_box_new();

    gtk_container_add (GTK_CONTAINER(event_box), opt_win->bgcolor);
    gtk_container_set_border_width (GTK_CONTAINER(event_box), 8);
    gtk_box_pack_start (GTK_BOX (hbox), event_box, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX(vbox), gtk_hseparator_new(), TRUE, TRUE, 0);
    
    /* Image save dir hbox: [ label | entry | button ] */
    hbox = gtk_hbox_new (FALSE, 8);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), 
			label_align_pad("Image Save Dir:", 1.0, 0.5, 8, 0),
			TRUE, TRUE, 0);

    gtk_box_pack_start (GTK_BOX (hbox),
			opt_win->entry_path = gtk_entry_new(),
			TRUE, TRUE, 0);

    if (opt_win->defaults.save_dir)
	gtk_entry_set_text (GTK_ENTRY (opt_win->entry_path),
			    opt_win->defaults.save_dir);
    
    gtk_box_pack_start (GTK_BOX (hbox),
			opt_win->browse = gtk_button_new_with_label ("..."),
			TRUE, TRUE, 0);

    gtk_tooltips_set_tip(opt_win->tooltips, opt_win->entry_path,
			 "Image save directory", NULL);

    attach_callback_tooltip (opt_win->browse, "clicked",
			     G_CALLBACK(on_options_dlg_browse),
			     opt_win,
			     CONNECT_NORMAL,
			     opt_win->tooltips, "Change image save directory");

    gtk_widget_add_events (event_box, GDK_BUTTON_PRESS_MASK);

    attach_callback_tooltip (event_box,
			     "button_press_event",
			     G_CALLBACK(on_options_dlg_change_bgcolor),
			     opt_win, CONNECT_NORMAL,
			     opt_win->tooltips, 
			     "Click to change background color");

    g_signal_connect (G_OBJECT (opt_win->bgcolor),
		      "expose_event",
		      G_CALLBACK (on_option_dlg_bgcolor_expose),
		      opt_win);    

    return opt_win;
}

