/* util.c
 *
 * Copyright (C) 2002 Ron Coscorrosa
 *
 * This software is released under the terms of the GNU General Public License
 * (GNU GPL), see the file COPYING for details.
 *
 * Misc. GTK utility functions
 *
 */

#include "util.h"
#include "config.h"

/* Returns a new gtk_menu filled with items from the list "items" */
GtkWidget*
menu_from_list (GList * items)
{
    int i;
    GtkWidget *menu;
    menu = gtk_menu_new();

    for (i = 0; i < g_list_length(items); ++i)
    {
	GtkWidget * menu_item = gtk_menu_item_new_with_label(
	    (char*)g_list_nth_data(items, i));
	gtk_widget_show (menu_item);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
    }
    return menu;
}



/* Returns a new label with the specified xalign/yalign and xpad/ypad */
GtkWidget*
label_align_pad (char * text, gfloat xalign, gfloat yalign,
		 gint xpad, gint ypad)
{
    GtkWidget * label = gtk_label_new (text);
    gtk_misc_set_alignment (GTK_MISC (label), xalign, yalign);
    gtk_misc_set_padding (GTK_MISC (label), xpad, ypad);
    return label;
}



/* Attaches a callback and tooltip to the widget w.  The first four arguments
 * are identical to those of g_signal_connect/g_signal_connect_swapped.
 *
 * type is either CONNECT_SWAPPED or CONNECT_NORMAL, which will determine
 * whether to use g_signal_connect_swapped or g_signal_connect.
 *
 * tooltips specifies the GtkTooltips to use, and tooltip is the actual text
 * for the tooltip.
 *
 * Any of the arguments, besides w, may be NULL in which case they are ignored.
 */
void
attach_callback_tooltip (GtkWidget * w,
			 char * sig_name,
			 GCallback func,
			 gpointer data,
			 int type,
			 GtkTooltips * tooltips,
			 char * tooltip)
{
    if (func && sig_name)
    {
	if (type == CONNECT_SWAPPED)
	    g_signal_connect_swapped (w, sig_name, func, data);
	else
	    g_signal_connect (G_OBJECT (w), sig_name, func, data);
    }
    if (tooltips && tooltip)
	gtk_tooltips_set_tip (tooltips, w, tooltip, NULL);
}



/* Returns all of the text in the gtk_textview "tv" */
char*
get_textview_text (GtkWidget * tv)
{
    GtkTextIter start, end;
    GtkTextBuffer * buffer;

    g_return_val_if_fail (GTK_IS_TEXT_VIEW (tv), NULL);

    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(tv));
    gtk_text_buffer_get_bounds (buffer, &start, &end);
    return (gtk_text_buffer_get_text (buffer, &start, &end, FALSE));
}



/* Sets the text of the gtk_textview "tv" */
void
set_textview_text (GtkWidget * tv, const char * text)
{
    GtkTextBuffer * buffer;

    g_return_if_fail (GTK_IS_TEXT_VIEW (tv));

    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(tv));
    gtk_text_buffer_set_text (buffer, text, -1);
}



/* Creates a new stock button, different from gtk_button_new_with_stock(), as
 * it allows you to specify the label and whether the image is on the left or
 * right.
 */
GtkWidget*
button_new_from_stock (char * stock_item, char * b_label,
		       gboolean image_first)
{
    GtkWidget *button = gtk_button_new();
    gtk_container_add (GTK_CONTAINER (button),
		       stock_image_label(stock_item, b_label, GTK_ICON_SIZE_BUTTON, 0, image_first));
    return button;
}



/* Returns a gtk_alignment which contains a stock image, and a label. */
GtkWidget*
stock_image_label (char * stock_item, char * b_label,
		   GtkIconSize size,
		   int spacing,
		   gboolean image_first)
{
    GtkWidget *image, *align, *hbox, *label = NULL;
    GtkWidget *first, *second;

    if (b_label)
	label = gtk_label_new_with_mnemonic (b_label);
    
    image = gtk_image_new_from_stock (stock_item, size);
    hbox = gtk_hbox_new (FALSE, 2);
    align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);

    /* I thought it might look interesting to have the image on the right
     * of the label as well... it looks like shit, but might as well leave this
     * in anyway... */
    first = (image_first || ! b_label) ? image : label;
    second = (b_label) ? ((image_first) ? label : image) : NULL;

    gtk_box_pack_start (GTK_BOX (hbox), first, FALSE, FALSE, 0);
    if (second)
	gtk_box_pack_end (GTK_BOX (hbox), second, FALSE, FALSE, 0);

    gtk_box_set_spacing (GTK_BOX(hbox), spacing);

    gtk_container_add (GTK_CONTAINER (align), hbox);
    return align;
}



/* Convenience function to attach a label to a table cell */
void table_attach_label (GtkTable * table, char * lbl_text,
			 int left, int right, int top, int bottom)
{
    g_return_if_fail (GTK_IS_TABLE (table));
    gtk_table_attach (GTK_TABLE (table),
		      label_align_pad (lbl_text, 1.0, 0.5, 8, 0),
		      left, right, top, bottom,
		      GTK_FILL,
		      0, 0, 0);
}



/* Clone of time(2), implemented w/ glib function for portability */
glong
glib_time()
{
    GTimeVal tv;
    g_get_current_time(&tv);
    return tv.tv_sec;
}



/* Opens "url" in whatever browser we detect first.  If no browser is found,
 * -1 is returned, 0 otherwise */
int
open_url (const char * url)
{
    int i, rc;
    GError * error = NULL;
    char * argv[3];

    /* There's 5! ways of ordering these, and I'm sure every one of them people
     * would bitch about, so I'm not going to worry about it... */
    const char * browsers [] = {
#ifdef WINDOWS
        "explorer",
#endif
	"mozilla",
	"konqueror",
	"galeon",
	"netscape",
	"opera"
    };
    
    argv[1] = (char *)url;
    argv[2] = NULL;

    for (i = 0; i < countof(browsers); ++i)
    {
	argv[0] = g_strdup(browsers[i]);
	rc = g_spawn_async (NULL,
			    argv,
			    NULL,
			    G_SPAWN_SEARCH_PATH,
			    NULL, NULL, NULL, &error);
	g_free (argv[0]);
	if (rc) return 0;
	if (error) g_error_free (error);
	error = NULL;
    }
    return -1;
}
